@@ -160,7 +160,7 @@ jtf_entry_keyrelease_cb(GtkWidget *w, GdkEventKey *evt, gpointer user_data)
         if(jtfdata->idle_id)
             g_source_remove(jtfdata->idle_id);
         jtf_get_words(jtfdata);
-        jtfdata->idle_id = g_idle_add(jtf_refilter_idled, jtfdata);
+        jtfdata->idle_id = g_timeout_add(500, jtf_refilter_idled, jtfdata);
     }
     
     return FALSE;
