@@ -228,11 +228,14 @@ xfmedia_keybindings_load(const gchar *file)
     gboolean ret = FALSE;
     
     if(!file) {
+        xfce_resource_push_path(XFCE_RESOURCE_CONFIG,
+                                SYSCONFDIR G_DIR_SEPARATOR_S "xdg");
         default_file = xfce_resource_lookup(XFCE_RESOURCE_CONFIG, KEYBIND_FILE);
+        xfce_resource_pop_path(XFCE_RESOURCE_CONFIG);
         file = default_file;
     }
     
-    if(!file || !g_file_test(file, G_FILE_TEST_EXISTS)) {
+    if(!file) {
         g_warning("No valid keybindings file found.");
         return FALSE;
     }
