@@ -70,6 +70,7 @@
 
 #include "main.h"
 #include "mainwin.h"
+#include <xfmedia/xfmedia-video-window.h>
 #include <xfmedia/xfmedia-playlist.h>
 #include <xfmedia/xfmedia-settings.h>
 #include "xfmedia-internal.h"
@@ -99,12 +100,6 @@ xfmedia_quit(XfmediaMainwin *mwin, XfmediaQuitMode quit_mode)
     xfmedia_remote_cleanup(mwin);
     xfmedia_plugins_unload();
     
-    if(xfmedia_settings_get_bool("/xfmedia/playlist/save_on_exit_always")
-            || !mwin->oneshot
-            || xfmedia_playlist_is_dirty(mwin->plist))
-    {
-        xfmedia_settings_save_playlist(mwin);
-    }
     xfmedia_settings_cleanup();
     
     /* save pointers */
@@ -414,7 +409,7 @@ main(int argc, char **argv)
     signal(SIGHUP, sighandler_cb);
     signal(SIGQUIT, sighandler_cb);
     signal(SIGUSR1, SIG_IGN);
-    signal(SIGUSR1, SIG_IGN);
+    signal(SIGUSR2, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
     
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
@@ -451,7 +446,7 @@ main(int argc, char **argv)
     
     DBG("calling XInitThreads()");
     if(!XInitThreads()) {
-        g_critical("XfmediaXine: XInitThreads() failed; thread-safe X11 libs unavailable!\n");
+        g_critical("XfmediaXine: XInitThreads() failed; thread-aware X11 libs unavailable!  Disabling video output.\n");
         video_port_id = "none";
     } else
         video_port_id = xfmedia_settings_get_string("/xfmedia/plugins/libxine/video_out");
@@ -468,9 +463,7 @@ main(int argc, char **argv)
     
     gdk_threads_enter();
     
-    video_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_title(GTK_WINDOW(video_window), _("Xfmedia Video"));
-    gtk_container_set_border_width(GTK_CONTAINER(video_window), 0);
+    video_window = xfmedia_video_window_new(NULL);
     gtk_widget_realize(video_window);
     if(odata.vwin_geom_flags & (VWIN_GEOM_X|VWIN_GEOM_Y)) {
         gtk_window_move(GTK_WINDOW(video_window),
@@ -478,24 +471,15 @@ main(int argc, char **argv)
                         odata.vwin_geom.y);
     }
     if(odata.vwin_geom_flags & (VWIN_GEOM_WIDTH|VWIN_GEOM_HEIGHT)) {
-        gtk_widget_set_size_request(video_window,
-                                    odata.vwin_geom.width,
-                                    odata.vwin_geom.height);
-        gtk_window_resize(GTK_WINDOW(video_window),
-                          odata.vwin_geom.width,
-                          odata.vwin_geom.height);
-        g_object_set_data(G_OBJECT(video_window),
-                          "vwin-size-fixed",
-                          GINT_TO_POINTER(0xfeedface));
+        xfmedia_video_window_set_fixed_size(XFMEDIA_VIDEO_WINDOW(video_window),
+                                            odata.vwin_geom.width,
+                                            odata.vwin_geom.height);
     }
     
     xfx = xfmedia_xine_new(video_port_id,
             xfmedia_settings_get_string("/xfmedia/plugins/libxine/audio_out"));
     gtk_widget_show(xfx);
-    gtk_container_add(GTK_CONTAINER(video_window), xfx);
-    DBG("about to realize...");
-    gtk_widget_realize(xfx);
-    DBG("    realize done");
+    xfmedia_video_window_set_video_widget(XFMEDIA_VIDEO_WINDOW(video_window), xfx);
     
     xfmedia_xine_engine_set_param(XFMEDIA_XINE(xfx),
             XINE_ENGINE_PARAM_VERBOSITY, odata.verbose_level);
@@ -507,7 +491,6 @@ main(int argc, char **argv)
             G_CALLBACK(vwin_destroy_cb), mwin);
     if(argc-next_arg > 11)
         defer = TRUE;
-    mwin->oneshot = FALSE;
     for(i = next_arg; i < argc; i++) {
         if(xfmedia_playlists_is_playlist_file(argv[i]))
             xfmedia_playlists_load(mwin->plist, argv[i]);
@@ -515,13 +498,14 @@ main(int argc, char **argv)
             gchar *title_utf8 = xfmedia_filename_to_name(argv[i]);
             xfmedia_playlist_append_entry(mwin->plist, title_utf8, -1, argv[i], FALSE);
             g_free(title_utf8);
-            mwin->oneshot = TRUE;
         }
         do_autoplay = TRUE;
     }
     
     if(next_arg >= argc && !odata.dont_load_playlist)
-        xfmedia_settings_load_playlist(mwin);
+        xfmedia_playlist_load_default_playlist(mwin->plist);
+    
+    xfmedia_playlist_set_dirty(mwin->plist, FALSE);
     
     do_startup_notify(mwin->window);
     
@@ -557,8 +541,10 @@ main(int argc, char **argv)
     mwin->session_id = xfmedia_remote_init(mwin);
     xfmedia_plugins_load(mwin);
     
-    if(odata.use_fullscreen)
-        xfmedia_xine_set_fullscreen(mwin->xfx, TRUE);
+    if(odata.use_fullscreen) {
+        xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(video_window),
+                                            TRUE);
+    }
     if(do_autoplay || odata.use_autoplay)
         g_idle_add(initial_autoplay, mwin);
     
