@@ -45,6 +45,10 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 
+#if LIBXFCEGUI4_CHECK_VERSION(4, 3, 5)
+#define xfce_about_dialog_new xfce_about_dialog_new_with_values
+#endif
+
 #define EXO_API_SUBJECT_TO_CHANGE
 #include <exo/exo.h>
 
@@ -52,6 +56,7 @@
 
 #include "xfmedia-xine.h"
 #include "mainwin.h"
+#include <xfmedia/xfmedia-video-window.h>
 #include <xfmedia/xfmedia-playlist.h>
 #include <xfmedia/xfmedia-playlist-queue.h>
 #include <xfmedia/xfmedia-settings.h>
@@ -319,9 +324,8 @@ xfmedia_mainwin_playlist_file_info_cb(GtkWidget *w, gpointer user_data)
     gtk_widget_show(frame);
     gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
     
-    lbl = exo_ellipsized_label_new(filename);
-    exo_ellipsized_label_set_ellipsize(EXO_ELLIPSIZED_LABEL(lbl),
-            EXO_PANGO_ELLIPSIZE_START);
+    lbl = gtk_label_new(filename);
+    gtk_label_set_ellipsize(GTK_LABEL(lbl), PANGO_ELLIPSIZE_START);
     gtk_label_set_selectable(GTK_LABEL(lbl), TRUE);
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
     gtk_label_set_text(GTK_LABEL(lbl), filename);
@@ -536,7 +540,13 @@ xfmedia_mainwin_playlist_file_info_cb(GtkWidget *w, gpointer user_data)
     entry = gtk_entry_new();
     if(xine_get_pos_length(mwin->extra_stream, &pos_stream, &pos_time, &length_time)) {
         length_time /= 1000;
-        g_snprintf(tmpbuf, TMPBUF, "%02d:%02d", length_time/60, length_time%60);
+        if(length_time >= 3600) {
+            g_snprintf(tmpbuf, TMPBUF, "%02d:%02d:%02d", length_time/3600,
+                       (length_time%3600)/60, (length_time%3600)%60);
+        } else {
+            g_snprintf(tmpbuf, TMPBUF, "%02d:%02d", length_time/60,
+                       length_time%60);
+        }
         gtk_entry_set_text(GTK_ENTRY(entry), tmpbuf);
         gtk_entry_set_width_chars(GTK_ENTRY(entry), strlen(tmpbuf));
     }
@@ -871,9 +881,7 @@ mainwin_add_url_dialog_response_cb(GtkWidget *dlg, gint response,
     gtk_widget_destroy(dlg);
     xfmedia_mainwin_enable_autohide(mwin);
     
-    /* yuck */
     g_signal_emit_by_name(G_OBJECT(mwin->plist), "playlist-scrolled", 0);
-    //xfmedia_playlist_treeview_scroll_cb(NULL, plist);
 }
 
 void
@@ -1193,6 +1201,8 @@ mainwin_add_dir_chooser_response_cb(GtkWidget *chooser, gint response,
     gchar *dirname;
     
     if(response == MY_FOLDER_SELECT_RESPONSE) {
+        gchar *last_dir;
+        
         dirname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
         if(dirname) {
             GtkWidget *combo = gtk_file_chooser_get_extra_widget(GTK_FILE_CHOOSER(chooser));
@@ -1203,9 +1213,11 @@ mainwin_add_dir_chooser_response_cb(GtkWidget *chooser, gint response,
             recurse_data->sort_type = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
             xfmedia_settings_set_int("/xfmedia/playlist/add_sort_type",
                     recurse_data->sort_type);
-            g_free(mwin->chooser_last_dir);
-            mwin->chooser_last_dir = g_strdup(dirname);
-            //plist->priv->is_dirty = TRUE;
+            
+            last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
+            xfmedia_settings_set_string("/xfmedia/general/chooser_last_dir", last_dir);
+            g_free(last_dir);
+            
             g_thread_create((GThreadFunc)recurse_add_dirs_th, recurse_data,
                     FALSE, NULL);
         }
@@ -1235,9 +1247,8 @@ xfmedia_mainwin_add_dir_cb(GtkWidget *w, gpointer user_data)
             GTK_WINDOW(mwin->window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN,
             MY_FOLDER_SELECT_RESPONSE, NULL);
-    if(mwin->chooser_last_dir)
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser),
-                mwin->chooser_last_dir);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser),
+            xfmedia_settings_get_string("/xfmedia/general/chooser_last_dir"));
     gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_ACCEPT);
     g_signal_connect(G_OBJECT(chooser), "response",
             G_CALLBACK(mainwin_add_dir_chooser_response_cb), mwin);
@@ -1273,6 +1284,8 @@ mainwin_add_file_chooser_response_cb(GtkWidget *chooser, gint response,
     }
     
     if(response == GTK_RESPONSE_ACCEPT || response == GTK_RESPONSE_APPLY) {
+        gchar *last_dir;
+        
         filenames = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(chooser));
         if(filenames && g_slist_length(filenames) < 10)
             defer = FALSE;
@@ -1290,13 +1303,10 @@ mainwin_add_file_chooser_response_cb(GtkWidget *chooser, gint response,
         }
         if(filenames)
             g_slist_free(filenames);
-        if(mwin->chooser_last_dir)
-            g_free(mwin->chooser_last_dir);
-        mwin->chooser_last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
-        //plist->priv->is_dirty = TRUE;
         
-        //if(nadded > 0)
-        //    xfmedia_playlist_shuffle_add_entries(mwin->plist, -1, nadded);
+        last_dir = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(chooser));
+        xfmedia_settings_set_string("/xfmedia/general/chooser_last_dir", last_dir);
+        g_free(last_dir);
     }
     
     if(response != GTK_RESPONSE_APPLY) {
@@ -1305,7 +1315,6 @@ mainwin_add_file_chooser_response_cb(GtkWidget *chooser, gint response,
     }
     if(response != GTK_RESPONSE_CANCEL)
         g_signal_emit_by_name(G_OBJECT(mwin->plist), "playlist-scrolled", 0);
-        //xfmedia_playlist_treeview_scroll_cb(NULL, plist);
 }
 
 void
@@ -1322,8 +1331,10 @@ xfmedia_mainwin_add_file_cb(GtkWidget *w, gpointer user_data)
     }
     
     toplevel = gtk_widget_get_toplevel(GTK_WIDGET(mwin->window));
-    chooser = gtk_file_chooser_dialog_new(_("Add Media File"), GTK_WINDOW(toplevel),
-            GTK_FILE_CHOOSER_ACTION_OPEN, NULL);
+    chooser = gtk_file_chooser_dialog_new(_("Add Media File"),
+                                          GTK_WINDOW(toplevel),
+                                          GTK_FILE_CHOOSER_ACTION_OPEN,
+                                          NULL, NULL);
     btn = xfmedia_custom_button_new(_("Add _Selected"), GTK_STOCK_ADD);
     gtk_widget_show(btn);
     gtk_dialog_add_action_widget(GTK_DIALOG(chooser), btn, GTK_RESPONSE_APPLY);
@@ -1333,9 +1344,8 @@ xfmedia_mainwin_add_file_cb(GtkWidget *w, gpointer user_data)
     gtk_widget_show(btn);
     gtk_dialog_add_action_widget(GTK_DIALOG(chooser), btn, GTK_RESPONSE_ACCEPT);
     gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
-    if(mwin->chooser_last_dir)
-        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser),
-                mwin->chooser_last_dir);
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(chooser),
+            xfmedia_settings_get_string("/xfmedia/general/chooser_last_dir"));
     gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_ACCEPT);
     g_signal_connect(G_OBJECT(chooser), "response",
             G_CALLBACK(mainwin_add_file_chooser_response_cb), mwin);
@@ -1698,7 +1708,6 @@ xfmedia_show_vis_menu_toggle_cb(GtkCheckMenuItem *mi, gpointer user_data)
     } else {
         if(!has_video && xfmedia_xine_get_status(mwin->xfx) == XINE_STATUS_PLAY) {
             gint width, height;
-            GdkGeometry geom;
             
             xfmedia_xine_set_vis(mwin->xfx,
                     xfmedia_settings_get_string("/xfmedia/general/selected_vis"));
@@ -1711,17 +1720,13 @@ xfmedia_show_vis_menu_toggle_cb(GtkCheckMenuItem *mi, gpointer user_data)
                 width = 480;
                 height = 360;
             }
-            geom.base_width = width;
-            geom.base_height = height;
-            geom.min_width = geom.min_height = 1;
-            geom.min_aspect = geom.max_aspect = (gdouble)width / height;
-            DBG("window size: %dx%d, aspect: %f", width, height, geom.max_aspect);
-            gtk_window_set_geometry_hints(GTK_WINDOW(mwin->video_window),
-                    GTK_WIDGET(mwin->xfx), &geom,
-                    GDK_HINT_MIN_SIZE|GDK_HINT_ASPECT|GDK_HINT_BASE_SIZE);
+            xfmedia_video_window_set_aspect(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                            width, height);
             gtk_widget_show(mwin->video_window);
-            gtk_window_resize(GTK_WINDOW(mwin->video_window),
-                    width, height);
+            if(!xfmedia_video_window_has_fixed_size(XFMEDIA_VIDEO_WINDOW(mwin->video_window))) {
+                gtk_window_resize(GTK_WINDOW(mwin->video_window),
+                                  width, height);
+            }
         }
     }
 }
@@ -1766,6 +1771,10 @@ xfmedia_autohidewin_enter_notify(GtkWidget *w, GdkEventCrossing *evt,
             gtk_window_unstick(GTK_WINDOW(mwin->window));
         if(xfmedia_settings_get_bool("/xfmedia/general/window_above"))
             gtk_window_set_keep_above(GTK_WINDOW(mwin->window), TRUE);
+        if(GTK_WIDGET_REALIZED(mwin->window)) {
+            gdk_x11_window_set_user_time(mwin->window->window,
+                                         gdk_x11_get_server_time(mwin->window->window));
+        }
         gtk_widget_show(mwin->window);
         gtk_widget_hide(mwin->autohide_window);
     }
@@ -1907,13 +1916,7 @@ xfmedia_mainwin_expander_activate_cb(GtkExpander *widget, gpointer user_data)
     }
 }
 
-static void
-about_dlg_zero_pointer(GtkWidget **dlg)
-{
-    *dlg = NULL;
-}
-
-static struct
+static const struct
 {
     gchar *name;
     gchar *email;
@@ -1928,12 +1931,15 @@ static struct
     { "lerouge", "lerouge@gmail.com", "fr" },
     { "Ankit Patel", "ankit644@yahoo.com", "gu" },
     { "Atilla Szerv\303\241c", "sas@321.hu", "hu" },
+    { "Daichi Kawahata", "daichi@xfce.org", "ja" },
     { "Rimas Kudelis", "rg@akl.lt", "lt" },
     { "Robert Kurowski", "koorek@o2.pl", "pl" },
     { "Tomas Schertel", "tschertel@gmail.com", "pt_BR" },
     { "Juraz Brosz", "juro@jurajbrosz.info", "sk" },
+    { "Maxim Dziumanenko", "mvd@softprom.kiev.ua", "uk" },
     { "Phan Vinh Thinh", "teppi@vnlinux.org", "vi" },
     { "Sheng Feng Zhou", "zhoushengfeng@highpoint-tech.com.cn", "zh_CN" },
+    { "Hydonsingore Cia", "hydonsingore@mail.educities.edu.tw", "zh_TW" },
     { NULL, NULL, NULL }
 };
 
@@ -1960,7 +1966,7 @@ xfmedia_about_cb(GtkWidget *w, gpointer user_data)
     ainfo = xfce_about_info_new("Xfmedia", VERSION " (" REVISION ")",
             _("Xfmedia is a lightweight media player,\nbased on the xine engine"),
             XFCE_COPYRIGHT_TEXT("2004-2005", "Brian Tarricone"), XFCE_LICENSE_GPL);
-    xfce_about_info_set_homepage(ainfo, "http://spuriousinterrupt.org/projects/xfmedia/");
+    xfce_about_info_set_homepage(ainfo, WEBSITE);
     
     xfce_about_info_add_credit(ainfo, "Brian Tarricone", "bjt23@cornell.edu",
             _("Original Author/Maintainer"));
@@ -1976,7 +1982,7 @@ xfmedia_about_cb(GtkWidget *w, gpointer user_data)
     g_signal_connect(G_OBJECT(dlg), "response",
             G_CALLBACK(gtk_widget_destroy), NULL);
     g_signal_connect_swapped(G_OBJECT(dlg), "destroy",
-            G_CALLBACK(about_dlg_zero_pointer), &dlg);
+            G_CALLBACK(xfmedia_widget_zero_pointer), &dlg);
     gtk_widget_show_all(dlg);
 }
 
@@ -1985,24 +1991,11 @@ xfmedia_vwin_button_press_cb(GtkWidget *w, GdkEventButton *evt,
         gpointer user_data)
 {
     XfmediaMainwin *mwin = user_data;
-    gint x, y;
     
     if(evt->button == 1 && evt->type == GDK_2BUTTON_PRESS) {
-        gboolean is_fs = xfmedia_xine_is_fullscreen(mwin->xfx);
-        
-        xfmedia_xine_set_fullscreen(mwin->xfx, !is_fs);
-        if(!is_fs) {
-            gtk_window_get_position(GTK_WINDOW(mwin->video_window), &x, &y);
-            xfmedia_settings_set_int("/xfmedia/general/vwin_pos_x", x);
-            xfmedia_settings_set_int("/xfmedia/general/vwin_pos_y", y);
-            gtk_widget_hide(mwin->video_window);
-        } else {
-            x = xfmedia_settings_get_int("/xfmedia/general/vwin_pos_x");
-            y = xfmedia_settings_get_int("/xfmedia/general/vwin_pos_y");
-            gtk_window_move(GTK_WINDOW(mwin->video_window), x, y);
-            gtk_widget_show(mwin->video_window);
-        }
-        
+        gboolean is_fs = xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window));
+        xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                            !is_fs);
         return TRUE;
     }
     
@@ -2015,32 +2008,33 @@ xfmedia_vwin_motion_notify_cb(GtkWidget *w, GdkEventMotion *evt,
 {
     XfmediaMainwin *mwin = user_data;
     GdkScreen *gscreen;
-    gint sheight, swidth;
+    gint monitor;
+    GdkRectangle rect;
     
-    if(!xfmedia_xine_is_fullscreen(mwin->xfx))
+    if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window)))
         return FALSE;
-    
+     
     gscreen = gtk_widget_get_screen(mwin->window);
-    swidth = gdk_screen_get_width(gscreen);
-    sheight = gdk_screen_get_height(gscreen);
+    monitor = gdk_screen_get_monitor_at_window(gscreen, w->window);
+    gdk_screen_get_monitor_geometry(gscreen, monitor, &rect);
     
-    if(!GTK_WIDGET_VISIBLE(mwin->fs_controls) && evt->y == sheight - 1) {
-        Window fs_window;
-        
-        fs_window = GDK_WINDOW_XWINDOW(xfmedia_xine_get_fullscreen_window(mwin->xfx));
-        gdk_property_change(mwin->fs_controls->window,
-                gdk_atom_intern("WM_TRANSIENT_FOR", FALSE),
-                gdk_atom_intern("WINDOW", FALSE),
-                32, GDK_PROP_MODE_REPLACE,
-                (const guchar *)&fs_window, sizeof(Window));
-        
-        gtk_widget_set_size_request(mwin->fs_controls, swidth, -1);
-        gtk_widget_realize(mwin->fs_controls);
-        gtk_window_move(GTK_WINDOW(mwin->fs_controls), 0,
-                sheight - mwin->fs_controls->allocation.height);
+#if 0
+    TRACE("entering: fscontrols: %s, evt->_rootx: %d, evt->y_root: %d, rect.height: %d",
+          GTK_WIDGET_VISIBLE(mwin->fs_controls)?"visible":"hidden",
+          (gint)evt->x_root, (gint)evt->y_root, rect.height);
+    DBG("fs window geom: %dx%d+%d+%d", w->allocation.width, w->allocation.height,
+        w->allocation.x, w->allocation.y);
+#endif
+    
+    if(!GTK_WIDGET_VISIBLE(mwin->fs_controls)
+       && evt->y_root >= rect.y + rect.height - 6)
+    {
+        gtk_widget_set_size_request(mwin->fs_controls, rect.width, -1);
         gtk_widget_show(mwin->fs_controls);
+        gtk_window_move(GTK_WINDOW(mwin->fs_controls), 0,
+                rect.y + rect.height - mwin->fs_controls->allocation.height);
     } else if(GTK_WIDGET_VISIBLE(mwin->fs_controls)
-            && evt->y < sheight - mwin->fs_controls->allocation.height)
+            && evt->y_root <= rect.y + rect.height - mwin->fs_controls->allocation.height - 6)
     {
         gtk_widget_hide(mwin->fs_controls);
     }
@@ -2162,6 +2156,7 @@ xfmedia_xfmedia_xine_progress_message_cb(XfmediaXine *xfx, const gchar *message,
         gpointer user_data)
 {
     XfmediaMainwin *mwin = user_data;
+    gchar text[32];
     
     DBG("got progress message: '%s' at %d%%", message, percent);
     
@@ -2173,6 +2168,8 @@ xfmedia_xfmedia_xine_progress_message_cb(XfmediaXine *xfx, const gchar *message,
         gtk_label_set_text(GTK_LABEL(mwin->progress_label), message);
         gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(mwin->progress_bar),
                 percent / 100.0);
+        g_snprintf(text, 32, "%d%%", percent);
+        gtk_progress_bar_set_text(GTK_PROGRESS_BAR(mwin->progress_bar), text);
     }
 }
 
@@ -2181,20 +2178,14 @@ xfmedia_xfmedia_xine_format_changed_cb(XfmediaXine *xfx, gint width, gint height
         gint aspect_code, gboolean is_pan_scan, gpointer user_data)
 {
     XfmediaMainwin *mwin = user_data;
-    GdkGeometry geom;
     
     DBG("got format changed signal: %dx%d, aspect_code: %d, is_pan_scan: %s",
             width, height, aspect_code, is_pan_scan?"yes":"no");
     
-    /*geom.base_width = width;
-    geom.base_height = height;*/
-    geom.min_width = geom.min_height = 1;
-    geom.min_aspect = geom.max_aspect = (gdouble)width / height;
-    DBG("window size: %dx%d, aspect: %f", width, height, geom.max_aspect);
-    gtk_window_set_geometry_hints(GTK_WINDOW(mwin->video_window),
-            GTK_WIDGET(mwin->xfx), &geom,
-            GDK_HINT_MIN_SIZE|GDK_HINT_ASPECT /*|GDK_HINT_BASE_SIZE*/);
-    gtk_window_resize(GTK_WINDOW(mwin->video_window), width, height);
+    xfmedia_video_window_set_aspect(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                    width, height);
+    if(!xfmedia_video_window_has_fixed_size(XFMEDIA_VIDEO_WINDOW(mwin->video_window)))
+        gtk_window_resize(GTK_WINDOW(mwin->video_window), width, height);
 }
 
 void
@@ -2319,27 +2310,12 @@ xfmedia_mainwin_keyrelease_cb(GtkWidget *w, GdkEventKey *evt, gpointer user_data
     return FALSE;
 }
 
-static void
-mainwin_popupmenu_deactivate_cb(GtkWidget *w, gpointer user_data)
-{
-    XfmediaMainwin *mwin = user_data;
-    guint sigid = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(w),
-            "xfmedia-deactivate-sigid"));
-    
-    g_signal_handler_disconnect(G_OBJECT(w), sigid);
-    xfmedia_mainwin_enable_autohide(mwin);
-}
-
 gboolean
 xfmedia_mainwin_menu_cb(GtkWidget *w, GdkEventButton *evt, gpointer user_data)
 {
     XfmediaMainwin *mwin = user_data;
     
     if(evt->button == 3 && mwin->popup_menu) {
-        guint sigid = g_signal_connect(G_OBJECT(mwin->popup_menu), "deactivate",
-                G_CALLBACK(mainwin_popupmenu_deactivate_cb), mwin);
-        g_object_set_data(G_OBJECT(mwin->popup_menu),
-                "xfmedia-deactivate-sigid", GUINT_TO_POINTER(sigid));
         xfmedia_mainwin_disable_autohide(mwin);
         gtk_menu_popup(GTK_MENU(mwin->popup_menu), NULL, NULL, NULL, NULL,
                 evt->button, evt->time);
@@ -2539,3 +2515,14 @@ xfmedia_mainwin_show_volume_window_cb(GtkToggleButton *tb, gpointer user_data)
                      NULL, NULL, GDK_CURRENT_TIME);
     gtk_grab_add(window);
 }
+
+gboolean
+xfmedia_mainwin_playlist_q_configure_cb(GtkWidget *w,
+                                        GdkEventConfigure *evt,
+                                        gpointer user_data)
+{
+    xfmedia_settings_set_int("/xfmedia/mediamarks/manager_x", evt->x);
+    xfmedia_settings_set_int("/xfmedia/mediamarks/manager_y", evt->y);
+    
+    return FALSE;
+}
