@@ -42,11 +42,11 @@
 
 #include "xfmedia-xine.h"
 #include "mainwin.h"
+#include <xfmedia/xfmedia-video-window.h>
 #include <xfmedia/xfmedia-settings.h>
 #include "trayicon.h"
 #include "keybindings.h"
 #include "main.h"
-#include "jumptofilewin.h"
 #include "xfmedia-internal.h"
 #include "mediamarks.h"
 #include "http.h"
@@ -56,11 +56,6 @@
 #define MAX_SHUFFLE_QUEUE_DEPTH 100
 #define AUTOHIDE_TIMEOUT        500
 
-enum {
-	DROP_TGT_REORDER = 0,
-	DROP_TGT_URILIST,
-	DROP_TGT_STRING
-};
 
 gboolean
 xfmedia_mainwin_check_do_autohide(XfmediaMainwin *mwin)
@@ -130,7 +125,7 @@ xfmedia_mainwin_check_do_autohide(XfmediaMainwin *mwin)
         gtk_widget_show(mwin->autohide_window);
         gtk_widget_hide(mwin->window);
         
-        if(xfmedia_xine_is_fullscreen(mwin->xfx))
+        if(xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window)))
             gdk_window_lower(mwin->autohide_window->window);
     }
     
@@ -159,13 +154,44 @@ xfmedia_mainwin_set_autohide(XfmediaMainwin *mwin, gboolean do_autohide)
 static void
 instantiate_extra_xine(XfmediaMainwin *mwin)
 {
+    xine_t *xine;
+    xine_cfg_entry_t cfgentry_a, cfgentry_v;
+    gint real_aud_bufsize = -1, real_vid_bufsize = -1;
+    
+    xine = xfmedia_xine_get_raw_xine_engine(mwin->xfx);
+    g_return_if_fail(xine);
+    
+    /* save buffer sizes */
+    if(xine_config_lookup_entry(xine, "video.num_buffers", &cfgentry_v)
+       && cfgentry_v.type == XINE_CONFIG_TYPE_NUM)
+    {
+        real_vid_bufsize = cfgentry_v.num_value;
+        cfgentry_v.num_value = 5;
+        xine_config_update_entry(xine, &cfgentry_v);
+    }
+    if(xine_config_lookup_entry(xine, "audio.num_buffers", &cfgentry_a)
+       && cfgentry_a.type == XINE_CONFIG_TYPE_NUM)
+    {
+        real_aud_bufsize = cfgentry_a.num_value;
+        cfgentry_a.num_value = 5;
+        xine_config_update_entry(xine, &cfgentry_a);
+    }
+    
     mwin->extra_xine_mx = g_mutex_new();
-    mwin->extra_vp = xine_open_video_driver(xfmedia_xine_get_raw_xine_engine(mwin->xfx),
-            "none", XINE_VISUAL_TYPE_NONE, NULL);
-    mwin->extra_ap = xine_open_audio_driver(xfmedia_xine_get_raw_xine_engine(mwin->xfx),
-            "none", NULL);
-    mwin->extra_stream = xine_stream_new(xfmedia_xine_get_raw_xine_engine(mwin->xfx),
-            mwin->extra_ap, mwin->extra_vp);
+    mwin->extra_vp = xine_open_video_driver(xine, "none",
+                                            XINE_VISUAL_TYPE_NONE, NULL);
+    mwin->extra_ap = xine_open_audio_driver(xine, "none", NULL);
+    mwin->extra_stream = xine_stream_new(xine, mwin->extra_ap, mwin->extra_vp);
+    
+    /* restore buffer sizes */
+    if(real_vid_bufsize != -1) {
+        cfgentry_v.num_value = real_vid_bufsize;
+        xine_config_update_entry(xine, &cfgentry_v);
+    }
+    if(real_aud_bufsize != -1) {
+        cfgentry_v.num_value = real_aud_bufsize;
+        xine_config_update_entry(xine, &cfgentry_a);
+    }
 }
 
 static void
@@ -201,15 +227,20 @@ static void
 mainwin_create_progress_box(XfmediaMainwin *mwin)
 {
     PangoFontDescription *fdesc;
+    gint size;
     
     mwin->progress_box = gtk_hbox_new(FALSE, BORDER/2);
+    gtk_container_set_border_width(GTK_CONTAINER(mwin->progress_box), 0);
     
     mwin->progress_label = gtk_label_new("");
     gtk_widget_show(mwin->progress_label);
     gtk_box_pack_start(GTK_BOX(mwin->progress_box), mwin->progress_label,
             FALSE, FALSE, 0);
     
-    fdesc = pango_font_description_from_string("italic");
+    fdesc = pango_font_description_copy(mwin->progress_label->style->font_desc);
+    pango_font_description_set_style(fdesc, PANGO_STYLE_ITALIC);
+    size = pango_font_description_get_size(fdesc);
+    pango_font_description_set_size(fdesc, size*PANGO_SCALE_LARGE);
     gtk_widget_modify_font(mwin->progress_label, fdesc);
     pango_font_description_free(fdesc);
     
@@ -226,9 +257,9 @@ mainwin_create_ui_msg_label(XfmediaMainwin *mwin)
 {
     PangoFontDescription *fdesc;
     
-    mwin->ui_message_label = exo_ellipsized_label_new("");
-    exo_ellipsized_label_set_ellipsize(EXO_ELLIPSIZED_LABEL(mwin->ui_message_label),
-            EXO_PANGO_ELLIPSIZE_END);
+    mwin->ui_message_label = gtk_label_new("");
+    gtk_label_set_ellipsize(GTK_LABEL(mwin->ui_message_label),
+                            PANGO_ELLIPSIZE_END);
     gtk_misc_set_alignment(GTK_MISC(mwin->ui_message_label), 0.0, 0.5);
     
     fdesc = pango_font_description_from_string("italic");
@@ -243,11 +274,10 @@ create_fs_control_window(XfmediaMainwin *mwin)
     gint iw, ih;
     guint opacity;
     
-    mwin->fs_controls = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    mwin->fs_controls = gtk_window_new(GTK_WINDOW_POPUP);
     gtk_window_set_title(GTK_WINDOW(mwin->fs_controls),
             "xfmedia-fullscreen-control-window");
     gtk_window_set_decorated(GTK_WINDOW(mwin->fs_controls), FALSE);
-    gtk_window_set_keep_above(GTK_WINDOW(mwin->fs_controls), TRUE);
     gtk_container_set_border_width(GTK_CONTAINER(mwin->fs_controls), BORDER/2);
     
     tophbox = gtk_hbox_new(FALSE, BORDER);
@@ -443,8 +473,6 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     mwin->xfx = xfx;
     g_signal_connect(G_OBJECT(xfx), "button-press-event",
             G_CALLBACK(xfmedia_vwin_button_press_cb), mwin);
-    g_signal_connect(G_OBJECT(xfx), "motion-notify-event",
-            G_CALLBACK(xfmedia_vwin_motion_notify_cb), mwin);
     g_signal_connect(G_OBJECT(xfx), "stream-ended",
             G_CALLBACK(xfmedia_xfmedia_xine_stream_ended_cb), mwin);
     g_signal_connect(G_OBJECT(xfx), "ui-message",
@@ -480,6 +508,7 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     
     mwin->window = topwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title(GTK_WINDOW(topwin), "Xfmedia");
+    gtk_window_set_role(GTK_WINDOW(topwin), "control/playlist window");
     gtk_container_set_border_width(GTK_CONTAINER(topwin), BORDER/2);
     if(xfmedia_settings_get_bool("/xfmedia/playlist/playlist_expanded")) {
         w = xfmedia_settings_get_int("/xfmedia/general/width");
@@ -559,9 +588,8 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     gtk_widget_show(lbl);
     gtk_container_add(GTK_CONTAINER(evtbox), lbl);
     
-    mwin->song_label = lbl = exo_ellipsized_label_new("");
-    exo_ellipsized_label_set_ellipsize(EXO_ELLIPSIZED_LABEL(lbl),
-            EXO_PANGO_ELLIPSIZE_END);
+    mwin->song_label = lbl = gtk_label_new("");
+    gtk_label_set_ellipsize(GTK_LABEL(lbl), PANGO_ELLIPSIZE_END);
     gtk_label_set_markup(GTK_LABEL(lbl), "<span size=\"large\"> </span>");
     gtk_label_set_use_markup(GTK_LABEL(lbl), TRUE);
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
@@ -725,15 +753,19 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     xfmedia_playlist_context_menu_prepend_item(mwin->plist, GTK_MENU_ITEM(mi));
     
     mi = gtk_image_menu_item_new_with_mnemonic(_("_Enqueue"));
+    img = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     xfmedia_playlist_context_menu_prepend_item(mwin->plist, GTK_MENU_ITEM(mi));
     g_signal_connect_swapped(G_OBJECT(mi), "activate",
                              G_CALLBACK(xfmedia_mainwin_queue_selected), mwin);
     
-    img = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
     mi = gtk_image_menu_item_new_with_mnemonic(_("Add _Mediamark..."));
-    gtk_widget_show(mi);
+    img = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    gtk_widget_show(mi);
     xfmedia_playlist_context_menu_prepend_item(mwin->plist, GTK_MENU_ITEM(mi));
     g_signal_connect(G_OBJECT(mi), "activate",
             G_CALLBACK(xfmedia_mainwin_playlist_add_mm_cb), mwin);
@@ -742,11 +774,11 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     gtk_widget_show(mi);
     xfmedia_playlist_context_menu_prepend_item(mwin->plist, GTK_MENU_ITEM(mi));
     
+    mi = gtk_image_menu_item_new_with_mnemonic(_("_File Info"));
     img = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_File Info"));
-    gtk_widget_show(mi);
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    gtk_widget_show(mi);
     xfmedia_playlist_context_menu_prepend_item(mwin->plist, GTK_MENU_ITEM(mi));
     g_signal_connect(G_OBJECT(mi), "activate",
             G_CALLBACK(xfmedia_mainwin_playlist_file_info_cb), mwin);
@@ -755,6 +787,9 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     
     mwin->popup_menu = menu = gtk_menu_new();
     gtk_widget_show(menu);
+    g_signal_connect_data(G_OBJECT(menu), "deactivate",
+                           G_CALLBACK(xfmedia_mainwin_enable_autohide), mwin,
+                           NULL, G_CONNECT_AFTER|G_CONNECT_SWAPPED);
     
     mi = gtk_image_menu_item_new_with_mnemonic(_("About _Xfmedia"));
     img = gtk_image_new_from_stock(GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU);
@@ -769,9 +804,9 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     
+    mi = gtk_image_menu_item_new_with_mnemonic(_("_Preferences"));
     img = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU);
     gtk_widget_show(img);
-    mi = gtk_image_menu_item_new_with_mnemonic(_("_Preferences"));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
@@ -807,6 +842,9 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     mwin->playlist_q_mi = mi;
                      
     mwin->mmarks_mi = mi = gtk_image_menu_item_new_with_mnemonic(_("_Mediamarks"));
+    img = gtk_image_new_from_stock(GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     menu2 = xfmedia_mediamarks_create_menu(mwin);
@@ -814,6 +852,9 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     xfmedia_mediamarks_hook_changed(xfmedia_mainwin_mediamarks_changed_cb, mwin);
     
     mi = gtk_image_menu_item_new_with_mnemonic(_("Re_order"));
+    img = gtk_image_new_from_stock(GTK_STOCK_SORT_ASCENDING, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
     gtk_widget_show(mi);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     
@@ -893,13 +934,16 @@ xfmedia_mainwin_new(XfmediaXine *xfx, GtkWindow *video_window)
     create_autohide_window(mwin);
     
     mwin->playlist_q = XFMEDIA_PLAYLIST_QUEUE(xfmedia_playlist_queue_new(GTK_WINDOW(mwin->window)));
-    gtk_widget_add_events(GTK_WIDGET(mwin->playlist_q), GDK_KEY_RELEASE);
+    gtk_widget_add_events(GTK_WIDGET(mwin->playlist_q), GDK_KEY_RELEASE
+                                                        |GDK_SUBSTRUCTURE_MASK);
     g_signal_connect(G_OBJECT(mwin->playlist_q), "key-release-event",
                      G_CALLBACK(xfmedia_mainwin_keyrelease_cb), mwin);
     g_signal_connect_swapped(G_OBJECT(mwin->playlist_q), "map-event",
                              G_CALLBACK(xfmedia_mainwin_disable_autohide), mwin);
     g_signal_connect_swapped(G_OBJECT(mwin->playlist_q), "unmap-event",
                              G_CALLBACK(xfmedia_mainwin_enable_autohide), mwin);
+    g_signal_connect(G_OBJECT(mwin->playlist_q), "configure-event",
+                     G_CALLBACK(xfmedia_mainwin_playlist_q_configure_cb), mwin);
     
     xfmedia_mainwin_set_time_label(mwin, 0, FALSE, NULL);
     
@@ -1007,19 +1051,39 @@ xfmedia_mainwin_set_time_label(XfmediaMainwin *mwin, gint time,
         gboolean is_remaining, const gchar *extra_pango_markup)
 {
     gchar time_lbl[128];
+    gint h, m, s;
     
-    g_snprintf(time_lbl, 128,
-            "<span font_family=\"monospace\" size=\"large\" weight=\"bold\" %s>%c%02d:%02d</span>",
-            extra_pango_markup ? extra_pango_markup : "",
-            is_remaining ? '-' : ' ',
-            time / 60, time % 60);
+    if(time >= 3600) {
+        h = time / 3600;
+        m = (time % 3600) / 60;
+        s = (time % 3600) % 60;
+        
+        g_snprintf(time_lbl, 128,
+                   "<span font_family=\"monospace\" size=\"large\" weight=\"bold\" %s>%c%02d:%02d:%02d</span>",
+                   extra_pango_markup ? extra_pango_markup : "",
+                   is_remaining ? '-' : ' ', h, m, s);
+    } else {
+        m = time / 60;
+        s = time % 60;
+        
+        g_snprintf(time_lbl, 128,
+                   "<span font_family=\"monospace\" size=\"large\" weight=\"bold\" %s>%c%02d:%02d</span>",
+                   extra_pango_markup ? extra_pango_markup : "",
+                   is_remaining ? '-' : ' ', m, s);
+    }
     gtk_label_set_markup(GTK_LABEL(mwin->time_label), time_lbl);
     
-    g_snprintf(time_lbl, 128,
-            "<span font_family=\"monospace\" %s>%c%02d:%02d</span>",
-            extra_pango_markup ? extra_pango_markup : "",
-            is_remaining ? '-' : ' ',
-            time / 60, time % 60);
+    if(time >= 3600) {
+        g_snprintf(time_lbl, 128,
+                   "<span font_family=\"monospace\" %s>%c%02d:%02d:%02d</span>",
+                   extra_pango_markup ? extra_pango_markup : "",
+                   is_remaining ? '-' : ' ', h, m, s);
+    } else {
+        g_snprintf(time_lbl, 128,
+                   "<span font_family=\"monospace\" %s>%c%02d:%02d</span>",
+                   extra_pango_markup ? extra_pango_markup : "",
+                   is_remaining ? '-' : ' ', m, s);
+    }
     gtk_label_set_markup(GTK_LABEL(mwin->fs_time_label), time_lbl);
 }
 
@@ -1420,7 +1484,7 @@ xfmedia_mainwin_play_uri(XfmediaMainwin *mwin, const gchar *uri, gint idx_ref)
     DBG("about to call xfmedia_xine_open()"); 
     if(xfmedia_xine_open(mwin->xfx, uri)) {
         gboolean was_visible = (GTK_WIDGET_VISIBLE(mwin->video_window)
-                || xfmedia_xine_is_fullscreen(mwin->xfx));
+                || xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window)));
         gboolean has_video = xfmedia_xine_get_stream_info(mwin->xfx,
                 XINE_STREAM_INFO_HAS_VIDEO);
         gboolean needs_vwin_show = FALSE;
@@ -1435,11 +1499,8 @@ xfmedia_mainwin_play_uri(XfmediaMainwin *mwin, const gchar *uri, gint idx_ref)
         if(!xfmedia_settings_get_bool("/xfmedia/general/show_audio_vis")
                 && was_visible && !has_video)
         {
-            if(xfmedia_xine_is_fullscreen(mwin->xfx)) {
-                if(GTK_WIDGET_VISIBLE(mwin->fs_controls))
-                    gtk_widget_hide(mwin->fs_controls);
-                xfmedia_xine_set_fullscreen(mwin->xfx, FALSE);
-            }
+            if(GTK_WIDGET_VISIBLE(mwin->fs_controls))
+                gtk_widget_hide(mwin->fs_controls);
             gtk_widget_hide(mwin->video_window);
         } else if(has_video) {
             if(!was_visible)
@@ -1473,12 +1534,8 @@ xfmedia_mainwin_play_uri(XfmediaMainwin *mwin, const gchar *uri, gint idx_ref)
             xfmedia_mainwin_set_song_label(mwin, name);
             g_free(name);
             
-            if(has_video
-               && !g_object_get_data(G_OBJECT(mwin->video_window),
-                                     "vwin-size-fixed"))
-            {
+            if(has_video) {
                 gint width, height;
-                GdkGeometry geom;
                 
                 width = xfmedia_xine_get_stream_info(mwin->xfx,
                         XINE_STREAM_INFO_VIDEO_WIDTH);
@@ -1489,18 +1546,14 @@ xfmedia_mainwin_play_uri(XfmediaMainwin *mwin, const gchar *uri, gint idx_ref)
                     height = 360;
                 }
                 
-                geom.base_width = width;
-                geom.base_height = height;
-                geom.min_width = geom.min_height = 1;
-                geom.min_aspect = geom.max_aspect = (gdouble)width / height;
-                DBG("window size: %dx%d, aspect: %f", width, height, geom.max_aspect);
-                gtk_window_set_geometry_hints(GTK_WINDOW(mwin->video_window),
-                        GTK_WIDGET(mwin->xfx), &geom,
-                        GDK_HINT_MIN_SIZE|GDK_HINT_ASPECT|GDK_HINT_BASE_SIZE);
-                
-                if(!needs_vwin_show) {
+                xfmedia_video_window_set_aspect(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                                width, height);
+                if(!xfmedia_video_window_has_fixed_size(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+                   && !xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+                   && needs_vwin_show)
+                {
                     gtk_window_resize(GTK_WINDOW(mwin->video_window),
-                            width, height);
+                                      width, height);                        
                 }
             }
             
@@ -1554,6 +1607,17 @@ mainwin_get_next_index(XfmediaMainwin *mwin, gint last_index)
         tot_items = xfmedia_playlist_get_n_entries(mwin->plist);
         cutoff = tot_items * 0.75;
         
+        if(tot_items == 0) {
+            /* this happened once; not sure why */
+            g_warning("tot_items == 0, checking again");
+            tot_items = xfmedia_playlist_get_n_entries(mwin->plist);
+            if(tot_items == 0) {
+                g_critical("tot_items STILL == 0");
+                tot_items = 1;
+            } else
+                g_warning("tot_items set properly on second try (%d)", tot_items);
+        }
+        
         do {
             if(attempts > 0)
                 DBG("shuffle collision!  trying again (attempt %d)", attempts);
@@ -1579,8 +1643,10 @@ mainwin_get_next_index(XfmediaMainwin *mwin, gint last_index)
                 next_id = 0;
             else {
                 xfmedia_xine_stop(mwin->xfx);
-                if(xfmedia_xine_is_fullscreen(mwin->xfx))
-                    xfmedia_xine_set_fullscreen(mwin->xfx, FALSE);
+                if(xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))) {
+                    xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                                        FALSE);
+                }
                 if(GTK_WIDGET_VISIBLE(mwin->video_window))
                     gtk_widget_hide(mwin->video_window);
                 next_id = -1;
@@ -1598,7 +1664,7 @@ xfmedia_mainwin_play_file_at_index(XfmediaMainwin *mwin, gint idx)
 {
     gchar *filename;
     gboolean ret = FALSE;
-    gint idx_start = -1, idx_end = -1;
+    guint idx_start = 0, idx_end = 0;
     
     if(idx == -1) {
         idx = mainwin_get_next_index(mwin, -1);
@@ -1718,7 +1784,6 @@ xfmedia_mainwin_toggle_playpause(XfmediaMainwin *mwin)
                     || xfmedia_settings_get_bool("/xfmedia/general/show_audio_vis"))
             {
                 gint width, height;
-                GdkGeometry geom;
                 
                 width = xfmedia_xine_get_stream_info(mwin->xfx,
                         XINE_STREAM_INFO_VIDEO_WIDTH);
@@ -1728,23 +1793,20 @@ xfmedia_mainwin_toggle_playpause(XfmediaMainwin *mwin)
                     width = 480;
                     height = 360;
                 }
-                geom.base_width = width;
-                geom.base_height = height;
-                geom.min_width = geom.min_height = 1;
-                geom.min_aspect = geom.max_aspect = (gdouble)width / height;
-                DBG("window size: %dx%d, aspect: %f", width, height, geom.max_aspect);
-                gtk_window_set_geometry_hints(GTK_WINDOW(mwin->video_window),
-                        GTK_WIDGET(mwin->xfx), &geom,
-                        GDK_HINT_MIN_SIZE|GDK_HINT_ASPECT|GDK_HINT_BASE_SIZE);
                 
-                if(!xfmedia_xine_is_fullscreen(mwin->xfx)
+                xfmedia_video_window_set_aspect(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                                width, height);
+                
+                if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
                         && !GTK_WIDGET_VISIBLE(mwin->video_window))
                 {
                     gtk_widget_show(mwin->video_window);
                 }
                 
-                gtk_window_resize(GTK_WINDOW(mwin->video_window),
-                        width, height);
+                if(!xfmedia_video_window_has_fixed_size(XFMEDIA_VIDEO_WINDOW(mwin->video_window))) {
+                    gtk_window_resize(GTK_WINDOW(mwin->video_window),
+                                      width, height);
+                }
                 
                 mainwin_set_tooltip_playing(mwin);
             }
@@ -1784,10 +1846,10 @@ xfmedia_mainwin_stop(XfmediaMainwin *mwin)
         
         if(GTK_WIDGET_VISIBLE(mwin->video_window))
             gtk_widget_hide(mwin->video_window);
-        if(xfmedia_xine_is_fullscreen(mwin->xfx)) {
+        if(xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))) {
             if(GTK_WIDGET_VISIBLE(mwin->fs_controls))
                 gtk_widget_hide(mwin->fs_controls);
-            xfmedia_xine_set_fullscreen(mwin->xfx, FALSE);
+            xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window), FALSE);
         }
         xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
     }
@@ -1905,9 +1967,14 @@ xfmedia_mainwin_disable_autohide(XfmediaMainwin *mwin)
 void
 xfmedia_mainwin_toggle_playlist_queue(XfmediaMainwin *mwin)
 {
-    if(!GTK_WIDGET_VISIBLE(GTK_WIDGET(mwin->playlist_q)))
+    if(!GTK_WIDGET_VISIBLE(GTK_WIDGET(mwin->playlist_q))) {
+        gint x = xfmedia_settings_get_int("/xfmedia/playlist_queue/pos_x");
+        gint y = xfmedia_settings_get_int("/xfmedia/playlist_queue/pos_y");
+        
+        if(x != -1 && y != -1)
+            gtk_window_move(GTK_WINDOW(mwin->playlist_q), x, y);
         gtk_widget_show(GTK_WIDGET(mwin->playlist_q));
-    else
+    } else
         gtk_widget_hide(GTK_WIDGET(mwin->playlist_q));
     
     g_signal_handlers_block_by_func(G_OBJECT(mwin->playlist_q_mi),
