@@ -93,7 +93,6 @@ typedef struct _XfmediaMainwin
     
     gboolean autohide_enabled;
     guint pos_id;
-    gboolean oneshot;
     XfmediaPlaylistEntryRef *cur_playing;
     gint cur_playing_length;
     GQueue *shuffle_history;
@@ -108,8 +107,6 @@ typedef struct _XfmediaMainwin
     xine_video_port_t *extra_vp;
     xine_audio_port_t *extra_ap;
     
-    gchar *chooser_last_dir;
-    
     /* settings dialog */
     GtkTreeView *ip_list;
     GtkTreeView *op_list;
