@@ -342,7 +342,7 @@ xfmedia_mediamarks_load(const gchar *filename)
     gint fd;
 #endif
     gchar *file_contents = NULL;
-    gint file_length;
+    guint file_length;
     GMarkupParser parser = {
         mediamarks_xml_start, mediamarks_xml_end, NULL, NULL, NULL
     };
@@ -1610,9 +1610,9 @@ manage_mediamarks_cb(GtkWidget *w, gpointer user_data)
     gtk_widget_show(toolbar);
     gtk_box_pack_start(GTK_BOX(topvbox), toolbar, FALSE, FALSE, 0);
     
-    img = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_SMALL_TOOLBAR);
+    img = gtk_image_new_from_stock(GTK_STOCK_NEW, GTK_ICON_SIZE_SMALL_TOOLBAR);
     gtk_widget_show(img);
-    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), _("Add Mediamark..."),
+    gtk_toolbar_append_item(GTK_TOOLBAR(toolbar), _("New Mediamark..."),
             _("Add a new mediamark"), NULL, img,
             G_CALLBACK(mmedit_addmm_cb), mdata);
     
