@@ -55,8 +55,11 @@
 
 #include <libxfce4util/libxfce4util.h>
 
+#define EXO_API_SUBJECT_TO_CHANGE 1
+#include <exo/exo.h>
+
 #include "playlist-files.h"
-#include <xfmedia/xfmedia-playlist.h>
+#include "xfmedia-common.h"
 
 static inline gboolean
 file_needs_dir(const gchar *filename)
@@ -80,7 +83,7 @@ string_strip_newlines(gchar *str)
 
 static gint
 xfmedia_playlists_load_flat_file(FILE *fp, const gchar *basedir,
-        XfMediaPlaylist *plist)
+        XfmediaPlaylist *plist)
 {
     gchar buf[PATH_MAX], buf1[PATH_MAX];
     gint nadded = 0;
@@ -107,7 +110,7 @@ xfmedia_playlists_load_flat_file(FILE *fp, const gchar *basedir,
 
 static gint
 xfmedia_playlists_load_m3u(FILE *fp, const gchar *basedir,
-        XfMediaPlaylist *plist)
+        XfmediaPlaylist *plist)
 {
     gchar buf[PATH_MAX], buf1[PATH_MAX], buf2[PATH_MAX], *p, *q;
     gint song_len, nadded = 0;
@@ -174,7 +177,7 @@ xfmedia_playlists_load_m3u(FILE *fp, const gchar *basedir,
 
 static gint
 xfmedia_playlists_load_pls(const gchar *filename, const gchar *basedir,
-        XfMediaPlaylist *plist)
+        XfmediaPlaylist *plist)
 {
     XfceRc *rcfile;
     gint i, nentries, length = 0;
@@ -233,7 +236,7 @@ xfmedia_playlists_load_pls(const gchar *filename, const gchar *basedir,
 
 typedef struct
 {
-    XfMediaPlaylist *plist;
+    XfmediaPlaylist *plist;
     const gchar *basedir;
     gint nadded;
     gboolean started;
@@ -336,7 +339,7 @@ asx_xml_text(GMarkupParseContext *context, const gchar *text, gsize text_len,
 
 static gint
 xfmedia_playlists_load_asx(FILE *fp, const gchar *basedir,
-        XfMediaPlaylist *plist)
+        XfmediaPlaylist *plist)
 {
     struct stat st;
 #ifdef HAVE_SYS_MMAN_H
@@ -407,11 +410,33 @@ xfmedia_playlists_load_asx(FILE *fp, const gchar *basedir,
 
 /***** end ASX parser *****/
 
+static gint
+xfmedia_playlists_load_wax(FILE *fp, const gchar *basedir,
+        XfmediaPlaylist *plist)
+{
+    gchar buf[4096];
+    gint nadded = 0;
+    
+    while(fgets(buf, 4096, fp) && g_ascii_strncasecmp(buf, "[reference]", 11))
+        ;
+        
+    while(fgets(buf, 4096, fp)) {
+        if(!g_ascii_strncasecmp(buf, "ref", 3)) {
+            gchar *title_utf8 = xfmedia_filename_to_name(buf);
+            xfmedia_playlist_append_entry(plist, title_utf8, -1, buf, FALSE);
+            nadded++;
+            g_free(title_utf8);
+        }
+    }
+    
+    return nadded;
+}
+
 
 /********** playlist file save functions **********/
 
 static void
-xfmedia_playlists_save_flat_file(XfMediaPlaylist *plist, FILE *fp)
+xfmedia_playlists_save_flat_file(XfmediaPlaylist *plist, FILE *fp)
 {
     gchar *filename = NULL;
     gint i, tot;
@@ -427,7 +452,7 @@ xfmedia_playlists_save_flat_file(XfMediaPlaylist *plist, FILE *fp)
 }
 
 static void
-xfmedia_playlists_save_m3u(XfMediaPlaylist *plist, FILE *fp)
+xfmedia_playlists_save_m3u(XfmediaPlaylist *plist, FILE *fp)
 {
     gchar *title = NULL, *filename = NULL;
     gint length = -1, i, tot;
@@ -455,7 +480,7 @@ xfmedia_playlists_save_m3u(XfMediaPlaylist *plist, FILE *fp)
 }
 
 static void
-xfmedia_playlists_save_pls(XfMediaPlaylist *plist, FILE *fp)
+xfmedia_playlists_save_pls(XfmediaPlaylist *plist, FILE *fp)
 {
     gchar *title = NULL, *filename = NULL;
     gint length = -1, i, tot;
@@ -484,19 +509,19 @@ xfmedia_playlists_save_pls(XfMediaPlaylist *plist, FILE *fp)
 
 
 gboolean
-xfmedia_playlists_load(XfMediaPlaylist *plist, const gchar *filename)
+xfmedia_playlists_load(XfmediaPlaylist *plist, const gchar *filename)
 {
     FILE *fp;
     gchar *p, *basedir;
-    gboolean ret = FALSE, is_m3u = FALSE, is_pls = FALSE, is_asx = FALSE;
+    gboolean ret = FALSE;
+    XfmediaPlaylistFormat format;
     gint nadded = 0;
     
-    if(g_str_has_suffix(filename, ".m3u") || g_str_has_suffix(filename, ".M3U"))
-        is_m3u = TRUE;
-    else if(g_str_has_suffix(filename, ".pls") || g_str_has_suffix(filename, ".PLS"))
-        is_pls = TRUE;
-    else if(g_str_has_suffix(filename, ".asx") || g_str_has_suffix(filename, ".ASX"))
-        is_asx = TRUE;
+    g_return_val_if_fail(filename && *filename, FALSE);
+    
+    format = xfmedia_playlists_query_playlist_format(filename);
+    if(format == XFMEDIA_PLAYLIST_FORMAT_UNKNOWN)
+        format = XFMEDIA_PLAYLIST_FORMAT_FLAT_FILE;
     
     fp = fopen(filename, "r");
     if(fp) {
@@ -506,16 +531,29 @@ xfmedia_playlists_load(XfMediaPlaylist *plist, const gchar *filename)
         else
             basedir = g_strdup(g_get_current_dir());
         
-        if(is_m3u)
-            nadded = xfmedia_playlists_load_m3u(fp, basedir, plist);
-        else if(is_pls) {
-            fclose(fp);
-            fp = NULL;
-            nadded = xfmedia_playlists_load_pls(filename, basedir, plist);
-        } else if(is_asx)
-            nadded = xfmedia_playlists_load_asx(fp, basedir, plist);
-        else
-            nadded = xfmedia_playlists_load_flat_file(fp, basedir, plist);
+        switch(format) {
+            case XFMEDIA_PLAYLIST_FORMAT_M3U:
+                nadded = xfmedia_playlists_load_m3u(fp, basedir, plist);
+                break;
+            
+            case XFMEDIA_PLAYLIST_FORMAT_PLS:
+                fclose(fp);
+                fp = NULL;
+                nadded = xfmedia_playlists_load_pls(filename, basedir, plist);
+                break;
+            
+            case XFMEDIA_PLAYLIST_FORMAT_ASX:
+                nadded = xfmedia_playlists_load_asx(fp, basedir, plist);
+                break;
+            
+            case XFMEDIA_PLAYLIST_FORMAT_WAX:
+                nadded = xfmedia_playlists_load_wax(fp, basedir, plist);
+                break;
+            
+            default:
+                nadded = xfmedia_playlists_load_flat_file(fp, basedir, plist);
+                break;
+        }
         
         g_free(basedir);
         
@@ -528,24 +566,43 @@ xfmedia_playlists_load(XfMediaPlaylist *plist, const gchar *filename)
 }
 
 gboolean
-xfmedia_playlists_save(XfMediaPlaylist *plist, const gchar *filename)
+xfmedia_playlists_save(XfmediaPlaylist *plist, const gchar *filename,
+        XfmediaPlaylistFormat format)
 {
     FILE *fp;
-    gboolean ret = FALSE, is_m3u = FALSE, is_pls = FALSE;
+    gboolean ret = FALSE;
     
-    if(g_str_has_suffix(filename, ".m3u") || g_str_has_suffix(filename, ".M3U"))
-        is_m3u = TRUE;
-    else if(g_str_has_suffix(filename, ".pls") || g_str_has_suffix(filename, ".PLS"))
-        is_pls = TRUE;
+    g_return_val_if_fail(filename && *filename, FALSE);
+    
+    if(format == XFMEDIA_PLAYLIST_FORMAT_UNKNOWN) {
+        if(g_str_has_suffix(filename, ".m3u") || g_str_has_suffix(filename, ".M3U"))
+            format = XFMEDIA_PLAYLIST_FORMAT_M3U;
+        else if(g_str_has_suffix(filename, ".pls") || g_str_has_suffix(filename, ".PLS"))
+            format = XFMEDIA_PLAYLIST_FORMAT_PLS;
+        else if(g_str_has_suffix(filename, ".asx") || g_str_has_suffix(filename, ".ASX"))
+            format = XFMEDIA_PLAYLIST_FORMAT_ASX;
+        else if(g_str_has_suffix(filename, ".wax") || g_str_has_suffix(filename, ".WAX"))
+            format = XFMEDIA_PLAYLIST_FORMAT_WAX;
+        else
+            format = XFMEDIA_PLAYLIST_FORMAT_FLAT_FILE;
+    }
     
     fp = fopen(filename, "w");
     if(fp) {
-        if(is_m3u)
-            xfmedia_playlists_save_m3u(plist, fp);
-        else if(is_pls)
-            xfmedia_playlists_save_pls(plist, fp);
-        else
-            xfmedia_playlists_save_flat_file(plist, fp);
+        switch(format) {
+            case XFMEDIA_PLAYLIST_FORMAT_M3U:
+                xfmedia_playlists_save_m3u(plist, fp);
+                break;
+            case XFMEDIA_PLAYLIST_FORMAT_PLS:
+                xfmedia_playlists_save_pls(plist, fp);
+                break;
+            case XFMEDIA_PLAYLIST_FORMAT_ASX:  /* no write support yet */
+            case XFMEDIA_PLAYLIST_FORMAT_WAX:  /* no write support yet */
+            case XFMEDIA_PLAYLIST_FORMAT_FLAT_FILE:
+            default:
+                xfmedia_playlists_save_flat_file(plist, fp);
+                break;
+        }
         
         fclose(fp);
         ret = TRUE;
@@ -558,9 +615,248 @@ gboolean
 xfmedia_playlists_is_playlist_file(const gchar *filename)
 {
     return (g_str_has_suffix(filename, ".m3u")
-            || g_str_has_suffix(filename, ".pls")
             || g_str_has_suffix(filename, ".M3U")
+            || g_str_has_suffix(filename, ".pls")
             || g_str_has_suffix(filename, ".PLS")
             || g_str_has_suffix(filename, ".asx")
-            || g_str_has_suffix(filename, ".ASX"));
+            || g_str_has_suffix(filename, ".ASX")
+            || g_str_has_suffix(filename, ".wax")
+            || g_str_has_suffix(filename, ".WAX"));
+}
+
+XfmediaPlaylistFormat
+xfmedia_playlists_query_playlist_format(const gchar *filename)
+{
+    g_return_val_if_fail(filename && *filename, XFMEDIA_PLAYLIST_FORMAT_UNKNOWN);
+    
+    if(g_str_has_suffix(filename, ".m3u") || g_str_has_suffix(filename, ".M3U"))
+        return XFMEDIA_PLAYLIST_FORMAT_M3U;
+    else if(g_str_has_suffix(filename, ".pls") || g_str_has_suffix(filename, ".PLS"))
+        return XFMEDIA_PLAYLIST_FORMAT_PLS;
+    else if(g_str_has_suffix(filename, ".asx") || g_str_has_suffix(filename, ".ASX"))
+        return XFMEDIA_PLAYLIST_FORMAT_ASX;
+    else if(g_str_has_suffix(filename, ".wax") || g_str_has_suffix(filename, ".WAX"))
+        return XFMEDIA_PLAYLIST_FORMAT_WAX;
+    
+    return XFMEDIA_PLAYLIST_FORMAT_UNKNOWN;
+}
+
+gchar *
+xfmedia_playlists_get_first_entry(const gchar *filename, gchar **title,
+        gint *stream_length)
+{
+    FILE *fp;
+    gchar *entry = NULL, buf[4096];
+    
+    if(g_str_has_suffix(filename, ".pls") || g_str_has_suffix(filename, ".PLS")) {
+        gboolean got_file = FALSE, got_title = FALSE, got_length = FALSE;
+        
+        fp = fopen(filename, "r");
+        if(!fp)
+            return NULL;
+        
+        if(!title)
+            got_title = TRUE;
+        if(!stream_length)
+            got_length = TRUE;
+        
+        while(fgets(buf, 4096, fp)) {
+            if(!got_file && !g_ascii_strncasecmp(buf, "file1=", 6)) {
+                while(buf[strlen(buf)-1] == '\n' || buf[strlen(buf)-1] == '\r')
+                    buf[strlen(buf)-1] = 0;
+                entry = g_strdup(buf+6);
+                got_file = TRUE;
+                continue;
+            }
+            
+            if(!got_title && !g_ascii_strncasecmp(buf, "title1=", 7)) {
+                while(buf[strlen(buf)-1] == '\n' || buf[strlen(buf)-1] == '\r')
+                    buf[strlen(buf)-1] = 0;
+                *title = g_strdup(buf+7);
+                got_title = TRUE;
+                continue;
+            }
+            
+            if(!got_length && !g_ascii_strncasecmp(buf, "length1=", 8)) {
+                while(buf[strlen(buf)-1] == '\n' || buf[strlen(buf)-1] == '\r')
+                    buf[strlen(buf)-1] = 0;
+                *stream_length = atoi(buf+8);
+                got_length = TRUE;
+            }
+            
+            if(got_file && got_title && got_length)
+                break;
+        }
+        
+        fclose(fp);
+    } else if(g_str_has_suffix(filename, ".m3u") || g_str_has_suffix(filename, ".M3U")) {
+        fp = fopen(filename, "r");
+        if(!fp)
+            return NULL;
+        
+        while(fgets(buf, 4096, fp)) {
+            if(!*buf || *buf == '\n' || *buf == '\r')
+                continue;
+            
+            if(!g_ascii_strncasecmp(buf, "#EXTM3U", 7))
+                continue;
+            
+            while(buf[strlen(buf)-1] == '\n' || buf[strlen(buf)-1] == '\r')
+                buf[strlen(buf)-1] = 0;
+            
+            if(!g_ascii_strncasecmp(buf, "#EXTINF:", 8)) {
+                gchar *p;
+                
+                if(title) {
+                    p = strstr(buf+8, ",");
+                    if(p)
+                        *title = g_strdup(p+1);
+                }
+                        
+                if(stream_length) {
+                    p = strstr(buf+8, ",");
+                    if(p) {
+                        *p = 0;
+                        *stream_length = atoi(buf+8);
+                    }
+                }
+                
+                continue;
+            }
+            
+            entry = g_strdup(buf);
+            break;
+        }
+        
+        fclose(fp);
+    } else if(g_str_has_suffix(filename, ".asx") || g_str_has_suffix(filename, ".ASX")) {
+        gboolean got_file = FALSE, got_title = FALSE;
+        
+        fp = fopen(filename, "r");
+        if(!fp)
+            return NULL;
+        
+        if(!title)
+            got_title = TRUE;
+        if(stream_length)
+            *stream_length = -1;
+        
+        while(fgets(buf, 4096, fp)) {
+            gchar *p, *q;
+            
+            if(!got_file && (p=strstr(buf, "<Ref href"))) {
+                p += 9;
+                while(*p && *p != '=')
+                    p++;
+                while(*p && *p != '"')
+                    p++;
+                p++;
+                if(*p) {
+                    q = strstr(p, "\"");
+                    if(q) {
+                        entry = g_strndup(p, q-p);
+                        got_file = TRUE;
+                        continue;
+                    }
+                }
+            }
+            
+            if(!got_title && (p=strstr(buf, "<Title>"))) {
+                p += 7;
+                q = strstr(p, "</Title>");
+                if(q) {
+                    *title = g_strndup(p, q-p);
+                    got_title = TRUE;
+                }
+            }
+            
+            if(got_file && got_title)
+                break;
+        }
+        
+        fclose(fp);
+    } else if(g_str_has_suffix(filename, ".wax") || g_str_has_suffix(filename, ".WAX")) {
+        fp = fopen(filename, "r");
+        
+        while(fgets(buf, 4096, fp) && g_ascii_strncasecmp(buf, "[reference]", 11))
+            ;
+        
+        while(fgets(buf, 4096, fp)) {
+            if(!g_ascii_strncasecmp(buf, "ref1=", 5)) {
+                while(buf[strlen(buf)-1] == '\n' || buf[strlen(buf)-1] == '\r')
+                    buf[strlen(buf)-1] = 0;
+                entry = g_strdup(buf+5);
+                break;
+            }
+        }
+        
+        fclose(fp);
+    }
+    
+    return entry;
+}
+
+static const gchar *cache_extensions[] = {
+    "asx",
+    "m3u",
+    "pls",
+    "wax",
+    NULL
+};
+
+/* FIXME: this function causes a double-free() crash */
+gchar *
+xfmedia_playlists_cache_find(const gchar *original_uri)
+{
+    gchar *filename, *md5sum, buf[512];
+    gint i;
+    
+    g_return_val_if_fail(original_uri && *original_uri, NULL);
+    
+    md5sum = exo_str_get_md5_str(original_uri);
+    
+    for(i = 0; cache_extensions[i]; i++) {
+        g_snprintf(buf, sizeof(buf), "xfmedia/cached-playlists/%s.%s", md5sum,
+                cache_extensions[i]);
+        g_free(md5sum);
+        filename = xfce_resource_save_location(XFCE_RESOURCE_CACHE, buf, FALSE);
+        if(!filename)
+            return NULL;
+        
+        if(g_file_test(filename, G_FILE_TEST_EXISTS))
+            return filename;
+        
+        g_free(filename);
+    }
+    
+    return NULL;
+}
+
+gchar *
+xfmedia_playlists_cache_save(const gchar *original_uri, const gchar *data,
+        const gchar *extension)
+{
+    gchar *filename, *md5sum, buf[512];
+    FILE *fp;
+    
+    g_return_val_if_fail(original_uri && *original_uri && data && *data, NULL);
+    
+    md5sum = exo_str_get_md5_str(original_uri);
+    
+    g_snprintf(buf, sizeof(buf), "xfmedia/cached-playlists/%s.%s", md5sum,
+            extension);
+    g_free(md5sum);
+    filename = xfce_resource_save_location(XFCE_RESOURCE_CACHE, buf, TRUE);
+    if(!filename)
+        return NULL;
+    
+    fp = fopen(filename, "w");
+    if(!fp) {
+        g_free(filename);
+        return NULL;
+    }
+    fwrite(data, 1, strlen(data), fp);
+    fclose(fp);
+    
+    return filename;
 }
