@@ -62,6 +62,8 @@
 #include "playlist-files.h"
 #include "jumptofilewin.h"
 
+#define PLAYLIST_CACHE_FILE  "xfmedia/playlist.m3u"
+
 enum {
     PLAYLIST_COL_NUM = 0,
     PLAYLIST_COL_TITLE,
@@ -138,7 +140,10 @@ struct _XfmediaPlaylistPriv
     
     gchar *allowed_extensions;
     gchar *chooser_last_dir;
-    gboolean is_dirty;
+    
+    guint autosave_id;
+    gboolean autosave_is_dirty;
+    gboolean usersave_is_dirty;
     
     XfmediaPlaylistEntryRef *bold_ref;
     XfmediaPlaylistEntryRef *italic_ref;
@@ -282,6 +287,9 @@ xfmedia_playlist_finalize(GObject *object)
     g_return_if_fail(XFMEDIA_IS_PLAYLIST(object));
     plist = XFMEDIA_PLAYLIST(object);
     
+    if(plist->priv->autosave_is_dirty)
+        xfmedia_playlist_save_default_playlist(plist);
+    
     g_free(plist->priv->allowed_extensions);
     g_free(plist->priv->shuffle_bitmap);
     
@@ -294,8 +302,35 @@ xfmedia_playlist_finalize(GObject *object)
     plist->priv = NULL;
     
     G_OBJECT_CLASS(parent_class)->finalize(object);
+} 
+
+static gboolean
+xfmedia_playlist_autosave_timeout(gpointer user_data)
+{
+    XfmediaPlaylist *plist = user_data;
+    
+    plist->priv->autosave_id = 0;
+    
+    if(!plist->priv->autosave_is_dirty)
+        return FALSE;
+    
+    xfmedia_playlist_save_default_playlist(plist);
+    
+    return FALSE;
 }
 
+static void
+xfmedia_playlist_schedule_autosave(XfmediaPlaylist *plist)
+{
+    if(plist->priv->autosave_id)
+        return;
+    
+    plist->priv->autosave_is_dirty = TRUE;
+    
+    plist->priv->autosave_id = g_timeout_add(7500,
+                                             xfmedia_playlist_autosave_timeout,
+                                             plist);
+}
 
 static void
 xfmedia_playlist_renumber_file_list(GtkListStore *ls, gint from_index,
@@ -474,7 +509,7 @@ xfmedia_playlist_go_up_cb(GtkWidget *w, gpointer user_data)
                     gtk_list_store_swap(plist->priv->file_list, &me, &above_me);
                     path_index = xfmedia_tree_path_to_index((GtkTreePath *)selected->data);
                     xfmedia_playlist_shuffle_swap_entries(plist, path_index, path_index-1);
-                    plist->priv->is_dirty = TRUE;
+                    xfmedia_playlist_schedule_autosave(plist);
                 }
             }
         }
@@ -517,7 +552,7 @@ xfmedia_playlist_go_down_cb(GtkWidget *w, gpointer user_data)
                 gtk_list_store_swap(plist->priv->file_list, &me, &below_me);
                 path_index = xfmedia_tree_path_to_index((GtkTreePath *)selected->data);
                 xfmedia_playlist_shuffle_swap_entries(plist, path_index, path_index+1);
-                plist->priv->is_dirty = TRUE;
+                xfmedia_playlist_schedule_autosave(plist);
             }
         }
         if(child_path)
@@ -551,23 +586,27 @@ xfmedia_playlist_new_cb(GtkWidget *w, gpointer user_data)
     GtkWidget *toplevel;
     gint resp;
     
-    if(plist->priv->is_dirty && xfmedia_playlist_get_n_entries(plist) > 0) {
+    if(plist->priv->usersave_is_dirty
+       && xfmedia_playlist_get_n_entries(plist) > 0)
+    {
         toplevel = gtk_widget_get_toplevel(GTK_WIDGET(plist));
         resp = playlist_dirty_confirm(GTK_WINDOW(toplevel));
         switch(resp) {
             case GTK_RESPONSE_CANCEL:
                 return;
             case GTK_RESPONSE_YES:
-                if(!xfmedia_playlist_save_as_cb(GTK_WIDGET(plist), plist))
+                if(xfmedia_playlist_save_as_cb(GTK_WIDGET(plist), plist))
+                    plist->priv->usersave_is_dirty = FALSE;
+                else
                     return;
                 break;
             case GTK_RESPONSE_NO:
+                plist->priv->usersave_is_dirty = FALSE;
                 break;
         }
     }
     
     xfmedia_playlist_clear(plist);
-    plist->priv->is_dirty = FALSE;
     xfmedia_playlist_shuffle_init_entries(plist, 0);
     
     g_signal_emit(G_OBJECT(plist), __signals[PLIST_SIG_CLEARED], 0);
@@ -659,16 +698,21 @@ xfmedia_playlist_open_cb(GtkWidget *w, gpointer user_data)
     
     topwin = gtk_widget_get_toplevel(w);
     
-    if(plist->priv->is_dirty && xfmedia_playlist_get_n_entries(plist) > 0) {
+    if(plist->priv->usersave_is_dirty
+       && xfmedia_playlist_get_n_entries(plist) > 0)
+    {
         resp = playlist_dirty_confirm(GTK_WINDOW(w));
         switch(resp) {
             case GTK_RESPONSE_CANCEL:
                 return;
             case GTK_RESPONSE_YES:
-                if(!xfmedia_playlist_save_as_cb(w, plist))
+                if(xfmedia_playlist_save_as_cb(w, plist))
+                    plist->priv->usersave_is_dirty = FALSE;
+                else
                     return;
                 break;
             case GTK_RESPONSE_NO:
+                plist->priv->usersave_is_dirty = FALSE;
                 break;
         }
     }
@@ -701,7 +745,7 @@ xfmedia_playlist_open_cb(GtkWidget *w, gpointer user_data)
         if(filename) {
             xfmedia_playlist_clear(plist);
             if(xfmedia_playlists_load(plist, filename)) {
-                plist->priv->is_dirty = FALSE;
+                xfmedia_playlist_schedule_autosave(plist);
                 g_signal_emit(G_OBJECT(plist), __signals[PLIST_SIG_OPENED], 0, filename);
             }
             g_free(filename);
@@ -1096,7 +1140,7 @@ xfmedia_playlist_treeview_drag_rcv_cb(GtkWidget *widget,
 								dest_index, src_index);
 					xfmedia_playlist_shuffle_swap_entries(plist, src_index, dest_index);
 					
-					plist->priv->is_dirty = TRUE;
+					xfmedia_playlist_schedule_autosave(plist);
 					drag_success = TRUE;
 				}
             } else {
@@ -1197,7 +1241,7 @@ xfmedia_playlist_treeview_drag_rcv_cb(GtkWidget *widget,
         
         drag_success = TRUE;
     } else if(data->target == _STRING) {
-        gchar *filename = g_strdup(data->data);
+        gchar *filename = g_strdup((gchar *)data->data);
         gint from_index = -1;
         
         if(gtk_tree_view_get_dest_row_at_pos(GTK_TREE_VIEW(plist->priv->treeview),
@@ -1466,11 +1510,13 @@ xfmedia_playlist_new()
     
     gtk_icon_size_lookup(GTK_ICON_SIZE_SMALL_TOOLBAR, &iw, &ih);
     
-    pix = xfce_icon_theme_load(icon_theme, "media-shuffle", iw);
+    pix = xfce_themed_icon_load("media-playlist-shuffle", iw);
     if(!pix)
-        pix = xfce_icon_theme_load(icon_theme, "stock_media-shuffle", iw);
+        pix = xfce_themed_icon_load("media-shuffle", iw);
     if(!pix)
-        pix = xfce_icon_theme_load(icon_theme, "xfmedia-shuffle", iw);
+        pix = xfce_themed_icon_load("stock_media-shuffle", iw);
+    if(!pix)
+        pix = xfce_themed_icon_load("xfmedia-shuffle", iw);
     if(pix) {
         img = gtk_image_new_from_pixbuf(pix);
         gtk_widget_show(img);
@@ -1485,11 +1531,13 @@ xfmedia_playlist_new()
             G_CALLBACK(xfmedia_playlist_shuffle_toggle_cb), plist);
     gtk_tooltips_set_tip(ttips, btn, _("Randomize play order"), NULL);
     
-    pix = xfce_icon_theme_load(icon_theme, "repeat", iw);
+    pix = xfce_themed_icon_load("media-playlist-repeat", iw);
     if(!pix)
-        pix = xfce_icon_theme_load(icon_theme, "stock_repeat", iw);
+        pix = xfce_themed_icon_load("repeat", iw);
     if(!pix)
-        pix = xfce_icon_theme_load(icon_theme, "xfmedia-repeat", iw);
+        pix = xfce_themed_icon_load("stock_repeat", iw);
+    if(!pix)
+        pix = xfce_themed_icon_load("xfmedia-repeat", iw);
     if(pix) {
         img = gtk_image_new_from_pixbuf(pix);
         gtk_widget_show(img);
@@ -1573,15 +1621,59 @@ xfmedia_playlist_new()
     return GTK_WIDGET(plist);
 }
 
+gboolean
+xfmedia_playlist_load_default_playlist(XfmediaPlaylist *plist)
+{
+    gchar *file;
+    gboolean ret = FALSE;
+    
+    /* use xfce_resource_lookup() so distros could provide a
+     * default playlist if they wanted to */
+    file = xfce_resource_lookup(XFCE_RESOURCE_CACHE, PLAYLIST_CACHE_FILE);
+    if(file) {
+        ret = xfmedia_playlists_load(plist, file);
+        g_free(file);
+    }
+    
+    return ret;
+}
+
+gboolean
+xfmedia_playlist_save_default_playlist(XfmediaPlaylist *plist)
+{
+    gchar *file;
+    gboolean ret = FALSE;
+    
+    file = xfce_resource_save_location(XFCE_RESOURCE_CACHE, PLAYLIST_CACHE_FILE,
+                                       TRUE);
+    if(file) {
+        ret = xfmedia_playlists_save(plist, file, XFMEDIA_PLAYLIST_FORMAT_M3U);
+        g_free(file);
+    }
+    
+    return ret;
+}
+
 static void
 xfmedia_playlist_insert_entry_internal(XfmediaPlaylist *plist, GtkTreeIter *itr,
         gint idx, const gchar *title, gint length, const gchar *filename,
         gboolean metadata_loaded)
 {
-    gchar length_str[32];
+    gchar length_str[32], *filename_new = NULL;
     gint last_mtime = 0;
     
-    if(length >= 0)
+    /* munge the filename for .iso files */
+    if((g_str_has_suffix(filename, ".iso")
+        || g_str_has_suffix(filename, ".ISO"))
+       && !g_str_has_prefix(filename, "dvd:"))
+    {
+        filename_new = g_strdup_printf("dvd:/%s", filename);
+    }
+    
+    if(length >= 3600) {
+        g_snprintf(length_str, 32, "%02d:%02d:%02d", length/3600,
+                   (length%3600)/60, (length%3600)%60);
+    } else if(length >= 0)
         g_snprintf(length_str, 32, "%02d:%02d", length/60, length%60);
     
     if(metadata_loaded) {
@@ -1594,7 +1686,7 @@ xfmedia_playlist_insert_entry_internal(XfmediaPlaylist *plist, GtkTreeIter *itr,
             PLAYLIST_COL_NUM, idx + 1,
             PLAYLIST_COL_TITLE, title,
             PLAYLIST_COL_LENGTH, length,
-            PLAYLIST_COL_FILENAME, filename,
+            PLAYLIST_COL_FILENAME, filename_new ? filename_new : filename,
             PLAYLIST_COL_LENGTH_S, (length >= 0) ? length_str : "??:??",
             PLAYLIST_COL_METADATA_LOADED, metadata_loaded,
             PLAYLIST_COL_LAST_MTIME, last_mtime,
@@ -1602,6 +1694,10 @@ xfmedia_playlist_insert_entry_internal(XfmediaPlaylist *plist, GtkTreeIter *itr,
     
     xfmedia_playlist_shuffle_add_entries(plist, idx, 1);
     g_signal_emit(G_OBJECT(plist), __signals[PLIST_SIG_ENTRY_ADDED], 0, idx);
+    
+    g_free(filename_new);
+    
+    xfmedia_playlist_schedule_autosave(plist);
 }
 
 guint
@@ -1656,7 +1752,13 @@ xfmedia_playlist_modify_entry(XfmediaPlaylist *plist, guint idx,
     }
     
     if(length >= 0) {
-        gchar *length_str = g_strdup_printf("%02d:%02d", length/60, length%60);
+        gchar *length_str;
+        
+        if(length >= 3600) {
+            length_str = g_strdup_printf("%02d:%02d:%02d", length/3600,
+                                         (length%3600)/60, (length%3600)%60);
+        } else
+            length_str = g_strdup_printf("%02d:%02d", length/60, length%60);
         gtk_list_store_set(plist->priv->file_list, &itr,
                 PLAYLIST_COL_LENGTH, length,
                 PLAYLIST_COL_LENGTH_S, length_str, -1);
@@ -1669,6 +1771,8 @@ xfmedia_playlist_modify_entry(XfmediaPlaylist *plist, guint idx,
     }
     
     g_signal_emit(G_OBJECT(plist), __signals[PLIST_SIG_ENTRY_CHANGED], 0, idx);
+    
+    xfmedia_playlist_schedule_autosave(plist);
 }
 
 gboolean
@@ -1697,6 +1801,8 @@ xfmedia_playlist_remove_entry(XfmediaPlaylist *plist, guint idx)
     if(filename)
         g_free(filename);
     
+    xfmedia_playlist_schedule_autosave(plist);
+    
     return TRUE;
 }
 
@@ -1707,6 +1813,8 @@ xfmedia_playlist_clear(XfmediaPlaylist *plist)
     
     gtk_list_store_clear(plist->priv->file_list);
     g_signal_emit(G_OBJECT(plist), __signals[PLIST_SIG_CLEARED], 0);
+    
+    xfmedia_playlist_schedule_autosave(plist);
 }
 
 
@@ -1771,6 +1879,28 @@ xfmedia_playlist_get_n_entries(XfmediaPlaylist *plist)
     return gtk_tree_model_iter_n_children(GTK_TREE_MODEL(plist->priv->file_list), NULL);
 }
 
+void
+xfmedia_playlist_set_dirty(XfmediaPlaylist *plist,
+                           gboolean is_dirty)
+{
+    g_return_if_fail(XFMEDIA_IS_PLAYLIST(plist));
+    
+    plist->priv->usersave_is_dirty = is_dirty;
+    plist->priv->autosave_is_dirty = is_dirty;
+    
+    if(!is_dirty && plist->priv->autosave_id) {
+        g_source_remove(plist->priv->autosave_id);
+        plist->priv->autosave_id = 0;
+    }
+}
+
+gboolean
+xfmedia_playlist_get_dirty(XfmediaPlaylist *plist)
+{
+    g_return_val_if_fail(XFMEDIA_IS_PLAYLIST(plist), FALSE);
+    return plist->priv->usersave_is_dirty;
+}
+
 gint
 xfmedia_playlist_get_selected(XfmediaPlaylist *plist)
 {
@@ -1809,6 +1939,10 @@ xfmedia_playlist_set_bold_entry(XfmediaPlaylist *plist,
         xfmedia_playlist_entry_ref_destroy(plist->priv->bold_ref);
     
     plist->priv->bold_ref = xfmedia_playlist_entry_ref_copy(ref);
+    
+    /* hack alert: work around gtk 2.6(?) oddity where the redraw doesn't
+     * happen unless you scroll */
+    gtk_widget_queue_draw(plist->priv->treeview);
 }
 void
 xfmedia_playlist_set_italic_entry(XfmediaPlaylist *plist,
@@ -1905,13 +2039,6 @@ xfmedia_playlist_context_menu_append_item(XfmediaPlaylist *plist,
             GTK_WIDGET(menu_item));
 }
 
-gboolean
-xfmedia_playlist_is_dirty(XfmediaPlaylist *plist)
-{
-    g_return_val_if_fail(XFMEDIA_IS_PLAYLIST(plist), FALSE);
-    return plist->priv->is_dirty;
-}
-
 void
 xfmedia_playlist_set_shuffle_state(XfmediaPlaylist *plist, gboolean state)
 {
