@@ -156,7 +156,7 @@ xfmedia_settings_create_devices_tab(XfmediaMainwin *mwin, GtkWidget *parent)
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
-    lbl = gtk_label_new_with_mnemonic(_("_CD Audio Device:"));
+    lbl = gtk_label_new_with_mnemonic(_("CD A_udio Device:"));
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
     gtk_size_group_add_widget(sg, lbl);
     gtk_widget_show(lbl);
@@ -178,7 +178,7 @@ xfmedia_settings_create_devices_tab(XfmediaMainwin *mwin, GtkWidget *parent)
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
-    lbl = gtk_label_new_with_mnemonic(_("_DVD Video Device:"));
+    lbl = gtk_label_new_with_mnemonic(_("DVD Vide_o Device:"));
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
     gtk_size_group_add_widget(sg, lbl);
     gtk_widget_show(lbl);
@@ -200,7 +200,7 @@ xfmedia_settings_create_devices_tab(XfmediaMainwin *mwin, GtkWidget *parent)
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
-    lbl = gtk_label_new_with_mnemonic(_("_Video CD Device:"));
+    lbl = gtk_label_new_with_mnemonic(_("Vid_eo CD Device:"));
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
     gtk_size_group_add_widget(sg, lbl);
     gtk_widget_show(lbl);
@@ -230,7 +230,7 @@ xfmedia_settings_create_devices_tab(XfmediaMainwin *mwin, GtkWidget *parent)
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
-    lbl = gtk_label_new_with_mnemonic(_("_Driver:"));
+    lbl = gtk_label_new_with_mnemonic(_("Dr_iver:"));
     gtk_misc_set_alignment(GTK_MISC(lbl), 0.0, 0.5);
     gtk_size_group_add_widget(sg, lbl);
     gtk_widget_show(lbl);
@@ -445,13 +445,6 @@ sd_destroy(struct SDData *sddata)
     sd_running = FALSE;
 }
 
-static void
-soea_toggle_cb(GtkToggleButton *tb, gpointer user_data)
-{
-    xfmedia_settings_set_bool("/xfmedia/playlist/save_on_exit_always",
-            !gtk_toggle_button_get_active(tb));
-}
-
 static void
 dock_autohide_cb(GtkToggleButton *tb, gpointer user_data)
 {
@@ -472,11 +465,11 @@ metadata_load_change_cb(GtkToggleButton *tb, gpointer user_data)
 }
 
 static void
-audio_vis_changed_cb(GtkMenuItem *mi, gpointer user_data)
+audio_vis_changed_cb(GtkComboBox *cb,
+                     gpointer user_data)
 {
     XfmediaMainwin *mwin = user_data;
-    GtkLabel *lbl = GTK_LABEL(gtk_bin_get_child(GTK_BIN(mi)));
-    const gchar *vis_name = gtk_label_get_text(lbl);
+    gchar *vis_name = gtk_combo_box_get_active_text(cb);
     
     xfmedia_settings_set_string("/xfmedia/general/selected_vis", vis_name);
     
@@ -486,6 +479,8 @@ audio_vis_changed_cb(GtkMenuItem *mi, gpointer user_data)
     {
         xfmedia_xine_set_vis(mwin->xfx, vis_name);
     }
+    
+    g_free(vis_name);
 }
 
 #if 0
@@ -586,7 +581,7 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
 {
     static GtkWidget *win = NULL;
     GtkWidget *topvbox, *vbox, *hbox, *btn, *frame, *frame_bin, *entry, *lbl,
-              *combo, *notebook, *menu, *mi, *cb;
+              *combo, *notebook, *menu, *cb;
     GtkTooltips *ttips;
     struct SDData *sddata;
     GList *vis_plugins;
@@ -674,16 +669,6 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
     gtk_tooltips_set_tip(ttips, btn,
             _("When a track ends, Xfmedia can automatically advance to the next track in the playlist and continue playback."), NULL);            
     
-    btn = gtk_check_button_new_with_mnemonic(_("_Don't save playlist on exit for one-shot invocations"));
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
-            !xfmedia_settings_get_bool("/xfmedia/playlist/save_on_exit_always"));
-    gtk_widget_show(btn);
-    gtk_box_pack_start(GTK_BOX(vbox), btn, FALSE, FALSE, 0);
-    g_signal_connect(G_OBJECT(btn), "toggled",
-            G_CALLBACK(soea_toggle_cb), mwin);
-    gtk_tooltips_set_tip(ttips, btn,
-            _("Only automatically save the playlist during interactive use; i.e., don't save the playlist when Xfmedia was invoked from the commandline with a short list of definite arguments.  This option needs to be named better."), NULL);
-    
     hbox = gtk_hbox_new(FALSE, BORDER);
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -741,7 +726,7 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
     g_signal_connect(G_OBJECT(btn), "toggled", G_CALLBACK(allws_cb), mwin);
     gtk_tooltips_set_tip(ttips, btn, _("Sets Xfmedia \"sticky\" such that it appears on all workspaces (virtual desktops)."), NULL);
     
-    btn = gtk_check_button_new_with_mnemonic(_("_Keep Xfmedia above other windows"));
+    btn = gtk_check_button_new_with_mnemonic(_("Keep X_fmedia above other windows"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
             xfmedia_settings_get_bool("/xfmedia/general/window_above"));
     gtk_widget_show(btn);
@@ -750,7 +735,7 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
     gtk_tooltips_set_tip(ttips, btn,
             _("Sets the Xfmedia window to \"Always-on-Top\" status."), NULL);
             
-    btn = gtk_check_button_new_with_mnemonic(_("_Dock and auto-hide Xfmedia at screen edges"));
+    btn = gtk_check_button_new_with_mnemonic(_("Dock and auto-_hide Xfmedia at screen edges"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn),
             xfmedia_settings_get_bool("/xfmedia/general/dock_autohide"));
     gtk_widget_show(btn);
@@ -834,6 +819,7 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
     menu = gtk_menu_new();
     gtk_widget_show(menu);
     
+    cb = gtk_combo_box_new_text();
     vis_plugins = xfmedia_xine_list_post_plugins_typed(mwin->xfx,
             XINE_POST_TYPE_AUDIO_VISUALIZATION);
     if(vis_plugins) {
@@ -843,22 +829,16 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
         for(l = vis_plugins; l; l = l->next) {
             if(!g_ascii_strcasecmp((const gchar *)l->data, selected_vis))
                 selected_vis_index = g_list_index(vis_plugins, l->data);
-            mi = gtk_menu_item_new_with_label((const gchar *)l->data);
-            gtk_widget_show(mi);
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-            g_signal_connect(G_OBJECT(mi), "activate",
-                    G_CALLBACK(audio_vis_changed_cb), mwin);
+            gtk_combo_box_append_text(GTK_COMBO_BOX(cb), (gchar *)l->data);
         }
         g_list_free(vis_plugins);
+        g_signal_connect(G_OBJECT(cb), "changed",
+                         G_CALLBACK(audio_vis_changed_cb), mwin);
     } else {
-        mi = gtk_menu_item_new_with_label(_("(none available)"));
-        gtk_widget_show(mi);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+        gtk_combo_box_append_text(GTK_COMBO_BOX(cb), _("(none available)"));
+        gtk_widget_set_sensitive(cb, FALSE);
     }
-    
-    cb = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(cb), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(cb), selected_vis_index);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(cb), selected_vis_index);
     gtk_widget_show(cb);
     gtk_box_pack_start(GTK_BOX(hbox), cb, TRUE, TRUE, 0);
     
@@ -904,5 +884,8 @@ xfmedia_settings_show_dialog(XfmediaMainwin *mwin)
     
     sd_running = TRUE;
     gtk_widget_show(win);
-
+    
+    xfmedia_mainwin_disable_autohide(mwin);
+    g_signal_connect_swapped(G_OBJECT(win), "destroy",
+                             G_CALLBACK(xfmedia_mainwin_enable_autohide), mwin);
 }
