@@ -62,7 +62,6 @@
 #include "mainwin.h"
 #include "playlist-files.h"
 
-#define PLAYLISTFILE             "xfmedia/playlist.m3u"
 #define XFMEDIA_CONFIG           "xfmedia/settings.xml"
 #define XFMEDIA_CONFIG_NEW       "xfmedia/settings.xml.new"
 #define XFMEDIA_CONFIG_WRITEFREQ 7500
@@ -580,6 +579,8 @@ xfmedia_settings_init()
     xfmedia_settings_add_bool("/xfmedia/general/show_tray_icon", FALSE);
     xfmedia_settings_add_bool("/xfmedia/general/minimize_to_tray", TRUE);
     xfmedia_settings_add_int("/xfmedia/general/tray_middle_click_action", 0);
+    xfmedia_settings_add_string("/xfmedia/general/chooser_last_dir",
+                                xfce_get_homedir());
     
 #if 0
     xfmedia_settings_add_int("/xfmedia/general/eq_band_0", 0);
@@ -603,56 +604,19 @@ xfmedia_settings_init()
     xfmedia_settings_add_bool("/xfmedia/playlist/shuffle", FALSE);
     xfmedia_settings_add_bool("/xfmedia/playlist/repeat", FALSE);
     xfmedia_settings_add_bool("/xfmedia/playlist/auto_advance", TRUE);
-    xfmedia_settings_add_bool("/xfmedia/playlist/save_on_exit_always", FALSE);
     xfmedia_settings_add_int("/xfmedia/playlist/metadata_load_impetus", 0);
     xfmedia_settings_add_bool("/xfmedia/playlist/repeat_one", FALSE);
     xfmedia_settings_add_int("/xfmedia/playlist/add_sort_type", 0);
     
+    xfmedia_settings_add_int("/xfmedia/playlist_queue/pos_x", -1);
+    xfmedia_settings_add_int("/xfmedia/playlist_queue/pos_y", -1);
+    
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_x", 25);
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_y", 25);
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_width", 615);
     xfmedia_settings_add_int("/xfmedia/mediamarks/manager_height", 400);
 }
 
-static inline gchar *
-get_default_playlist_file()
-{
-    gchar *playlist_file;
-    
-    playlist_file = xfce_resource_save_location(XFCE_RESOURCE_CACHE,
-            PLAYLISTFILE, TRUE);
-    
-    return playlist_file;
-}    
-
-gboolean
-xfmedia_settings_load_playlist(XfmediaMainwin *mwin)
-{
-    gchar *file = get_default_playlist_file();
-    gboolean ret = FALSE;
-    
-    if(file) {
-        ret = xfmedia_playlists_load(mwin->plist, file);
-        g_free(file);
-    }
-    
-    return ret;
-}
-
-gboolean
-xfmedia_settings_save_playlist(XfmediaMainwin *mwin)
-{
-    gchar *file = get_default_playlist_file();
-    gboolean ret = FALSE;
-    
-    if(file) {
-        ret = xfmedia_playlists_save(mwin->plist, file, XFMEDIA_PLAYLIST_FORMAT_M3U);
-        g_free(file);
-    }
-    
-    return ret;
-}
-
 
 /* XML file parser */
 
