@@ -101,12 +101,42 @@ xfmedia_tray_icon_button_press_cb(GtkWidget *w, GdkEventButton *evt,
                 break;
         }
     } else if(evt->button == 3) {
-        if(tray_icon->menu) {
+      if(tray_icon->menu) {
             gtk_menu_popup(GTK_MENU(tray_icon->menu), NULL, NULL, NULL,
                     NULL, evt->button, evt->time);
         }
         return TRUE;
     }
+    return FALSE;
+}
+
+static gboolean
+xfmedia_tray_icon_button_scroll_cb(GtkWidget *w,
+                                   GdkEventScroll *evt,
+                                   gpointer user_data)
+{
+    XfmediaTrayIcon *tray_icon = user_data;
+    XfmediaMainwin *mwin = tray_icon->mwin;
+    
+    if(evt->direction == GDK_SCROLL_UP) {
+        gint volume = xfmedia_xine_get_param(mwin->xfx,
+                                             XINE_PARAM_AUDIO_VOLUME);
+        volume += 2;
+        if(volume > 100)
+	      volume = 100;
+        xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME, volume);
+
+        return TRUE;
+    } else if(evt->direction == GDK_SCROLL_DOWN) {
+        gint volume = xfmedia_xine_get_param(mwin->xfx,
+                                             XINE_PARAM_AUDIO_VOLUME);
+        volume -= 2;
+        if(volume < 0)
+	        volume = 0;
+        xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME, volume);
+
+        return TRUE;
+    }
     
     return FALSE;
 }
@@ -139,7 +169,7 @@ _xfmedia_tray_icon_create(GdkScreen *gscreen, gpointer cb_data)
     gtk_container_add(GTK_CONTAINER(tray_icon), evtbox);
     
     gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &iw, &ih);
-    pix = xfce_icon_theme_load(icon_theme, "xfmedia", iw);
+    pix = xfce_themed_icon_load("xfmedia", iw);
     if(!pix) {
         tmp = gdk_pixbuf_from_pixdata(&xfmedia_icon, TRUE, NULL);
         if(tmp && gdk_pixbuf_get_width(tmp) == iw && gdk_pixbuf_get_height(tmp) == ih)
@@ -159,9 +189,11 @@ _xfmedia_tray_icon_create(GdkScreen *gscreen, gpointer cb_data)
     gtk_container_add(GTK_CONTAINER(evtbox), img);
     
     g_signal_connect(G_OBJECT(evtbox), "button-press-event",
-            G_CALLBACK(xfmedia_tray_icon_button_press_cb), cb_data);
+                     G_CALLBACK(xfmedia_tray_icon_button_press_cb), cb_data);
+    g_signal_connect(G_OBJECT(evtbox), "scroll-event",
+                     G_CALLBACK(xfmedia_tray_icon_button_scroll_cb), cb_data);
     g_signal_connect(G_OBJECT(tray_icon), "destroy",
-            G_CALLBACK(xfmedia_tray_icon_destroy_cb), cb_data);
+                     G_CALLBACK(xfmedia_tray_icon_destroy_cb), cb_data);
     
     gtk_widget_show(tray_icon);
     
