@@ -27,6 +27,7 @@
 
 #include "xfmedia-common.h"
 #include "mainwin.h"
+#include <xfmedia/xfmedia-video-window.h>
 #include <xfmedia/xfmedia-playlist.h>
 #include "xfmedia-xine.h"
 
@@ -228,7 +229,7 @@ xfmedia_get_fullscreen(XfmediaPlugin *plugin)
     
     mwin = g_object_get_data(G_OBJECT(plugin), "__xfmedia-mainwin");
     
-    return xfmedia_xine_is_fullscreen(mwin->xfx);
+    return xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window));
 }
 
 void
@@ -240,7 +241,8 @@ xfmedia_set_fullscreen(XfmediaPlugin *plugin, gboolean setting)
     
     mwin = g_object_get_data(G_OBJECT(plugin), "__xfmedia-mainwin");
     
-    xfmedia_xine_set_fullscreen(mwin->xfx, setting);
+    xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                        setting);
 }
 
 gboolean
@@ -252,7 +254,7 @@ xfmedia_get_video_window_visible(XfmediaPlugin *plugin)
     
     mwin = g_object_get_data(G_OBJECT(plugin), "__xfmedia-mainwin");
     
-    return (xfmedia_xine_is_fullscreen(mwin->xfx)
+    return (xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
             || GTK_WIDGET_VISIBLE(mwin->video_window));
 }
 
@@ -265,9 +267,9 @@ xfmedia_set_video_window_visible(XfmediaPlugin *plugin, gboolean setting)
     
     mwin = g_object_get_data(G_OBJECT(plugin), "__xfmedia-mainwin");
     
-    if(!setting && xfmedia_xine_is_fullscreen(mwin->xfx))
-        xfmedia_xine_set_fullscreen(mwin->xfx, FALSE);
-    else if(!xfmedia_xine_is_fullscreen(mwin->xfx)) {
+    if(!setting && xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window)))
+        xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window), FALSE);
+    else if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))) {
         if(setting)
             gtk_widget_show(mwin->video_window);
         else
