@@ -90,12 +90,11 @@ void xfmedia_mainwin_sort_title_cb(GtkWidget *w, gpointer user_data);
 void xfmedia_mainwin_sort_filename_cb(GtkWidget *w, gpointer user_data);
 void xfmedia_mainwin_randomise_playlist_cb(GtkWidget *w, gpointer user_data);
 void xfmedia_mainwin_show_volume_window_cb(GtkToggleButton *tb, gpointer user_data);
+gboolean xfmedia_mainwin_playlist_q_configure_cb(GtkWidget *w, GdkEventConfigure *evt, gpointer user_data);
 
 /* settings.c */
 void xfmedia_settings_init();
 void xfmedia_settings_cleanup();
-gboolean xfmedia_settings_load_playlist(XfmediaMainwin *mwin);
-gboolean xfmedia_settings_save_playlist(XfmediaMainwin *mwin);
 void xfmedia_settings_show_dialog(XfmediaMainwin *mwin);
 
 /* xfmedia-plugins.c */
