@@ -36,6 +36,7 @@
 #include "xfmedia-internal.h"
 #include "main.h"
 #include "mainwin.h"
+#include <xfmedia/xfmedia-video-window.h>
 #include "xfmedia-xine.h"
 #include "jumptofilewin.h"
 #include "keybindings.h"
@@ -85,24 +86,20 @@ KEYBIND_FUNC(kb_quit)
 
 KEYBIND_FUNC(kb_fullscreen)
 {
-    gboolean is_fs = xfmedia_xine_is_fullscreen(mwin->xfx);
+    gboolean is_fs = xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window));
     gint x, y;
     
-    if(!is_fs && !GTK_WIDGET_VISIBLE(mwin->video_window))
+    if(!GTK_WIDGET_VISIBLE(mwin->video_window))
         return;
     
-    xfmedia_xine_set_fullscreen(mwin->xfx, !is_fs);
     if(!is_fs) {
         gtk_window_get_position(GTK_WINDOW(mwin->video_window), &x, &y);
         xfmedia_settings_set_int("/xfmedia/general/vwin_pos_x", x);
         xfmedia_settings_set_int("/xfmedia/general/vwin_pos_y", y);
-        gtk_widget_hide(mwin->video_window);
-    } else {
-        x = xfmedia_settings_get_int("/xfmedia/general/vwin_pos_x");
-        y = xfmedia_settings_get_int("/xfmedia/general/vwin_pos_y");
-        gtk_window_move(GTK_WINDOW(mwin->video_window), x, y);
-        gtk_widget_show(mwin->video_window);
     }
+    
+    xfmedia_video_window_set_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window),
+                                        !is_fs);
 }
 
 KEYBIND_FUNC(kb_repeat)
@@ -359,8 +356,11 @@ KEYBIND_FUNC(kb_zoom_in)
 {
     gint cur_zoom;
     
-    if(!xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+       || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    {
         return;
+    }
     
     cur_zoom = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X);
     
@@ -376,8 +376,11 @@ KEYBIND_FUNC(kb_zoom_out)
 {
     gint cur_zoom;
     
-    if(!xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+       || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    {
         return;
+    }
     
     cur_zoom = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_VO_ZOOM_X);
     
@@ -393,8 +396,11 @@ KEYBIND_FUNC(kb_vwin_larger)
 {
     gint w = -1, h = -1;
     
-    if(xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+       || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    {
         return;
+    }
     
     gtk_window_get_size(GTK_WINDOW(mwin->video_window), &w, &h);
     if(w <= 0 || h <= 0)
@@ -407,8 +413,11 @@ KEYBIND_FUNC(kb_vwin_smaller)
 {
     gint w = -1, h = -1;
     
-    if(xfmedia_xine_is_fullscreen(mwin->xfx) || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    if(!xfmedia_video_window_get_fullscreen(XFMEDIA_VIDEO_WINDOW(mwin->video_window))
+       || !GTK_WIDGET_VISIBLE(mwin->video_window))
+    {
         return;
+    }
     
     gtk_window_get_size(GTK_WINDOW(mwin->video_window), &w, &h);
     if(w <= 0 || h <= 0)
@@ -439,6 +448,63 @@ KEYBIND_FUNC(kb_vol_down)
     xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_VOLUME, volume);
 }
 
+KEYBIND_FUNC(kb_aud_next)
+{
+    gint cur, max;
+    
+    max = xfmedia_xine_get_stream_info(mwin->xfx,
+                                       XINE_STREAM_INFO_MAX_AUDIO_CHANNEL);
+    cur = xfmedia_xine_get_param(mwin->xfx,
+                                 XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
+    
+    DBG("setting audio track to %d, max is %d", cur+1, max);
+    
+    if(++cur > max)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, cur);
+}
+
+KEYBIND_FUNC(kb_aud_prev)
+{
+    gint cur;
+    
+    cur = xfmedia_xine_get_param(mwin->xfx,
+                                 XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
+    
+    if(--cur < -2)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, cur);
+}
+
+KEYBIND_FUNC(kb_sub_next)
+{
+    gint cur, max;
+    
+    max = xfmedia_xine_get_stream_info(mwin->xfx,
+                                       XINE_STREAM_INFO_MAX_SPU_CHANNEL);
+    cur = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_SPU_CHANNEL);
+    
+    if(++cur > max)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_SPU_CHANNEL, cur);
+}
+
+KEYBIND_FUNC(kb_sub_prev)
+{
+    gint cur;
+    
+    cur = xfmedia_xine_get_param(mwin->xfx, XINE_PARAM_SPU_CHANNEL);
+    
+    if(--cur < 0)
+        return;
+    
+    xfmedia_xine_set_param(mwin->xfx, XINE_PARAM_SPU_CHANNEL, cur);
+}
+
+
 static XfmediaKeybindEntry keybind_entries[] =
 {
     { N_("Play"), "play", kb_play, NULL },
@@ -476,7 +542,11 @@ static XfmediaKeybindEntry keybind_entries[] =
     { N_("Video Window Larger"), "vid-win-larger", kb_vwin_larger, NULL },
     { N_("Video Window Smaller"), "vid-win-smaller", kb_vwin_smaller, NULL },
     { N_("Volume Up"), "vol-up", kb_vol_up, NULL },
-    { N_("Volume Down"), "vol-down", kb_vol_down, NULL }
+    { N_("Volume Down"), "vol-down", kb_vol_down, NULL },
+    { N_("Audio Track Next"), "aud-next", kb_aud_next, NULL },
+    { N_("Audio Track Previous"), "aud-prev", kb_aud_prev, NULL },
+    { N_("Subtitle Track Next"), "sub-next", kb_sub_next, NULL, },
+    { N_("Subtitle Track Previous"), "sub-prev", kb_sub_prev, NULL },
 };
 
 void
