@@ -23,6 +23,8 @@
 
 #include <gmodule.h>
 
+#include <libxfce4util/libxfce4util.h>
+
 #include <xfmedia/xfmedia-plugin.h>
 #include "mainwin.h"
 
@@ -459,21 +461,22 @@ static GList *xfmedia_plugins = NULL;
 void
 xfmedia_plugins_load(gpointer mwin)  /* gpointer to avoid an include */
 {
-    gchar **plugin_dirs, fullpath[PATH_MAX];
-    gint i;
+    gchar *plugin_dirs[N_PLUGIN_DIRS], fullpath[PATH_MAX];
+    gint i = 0;
     const gchar *file;
     
-    plugin_dirs = g_malloc(sizeof(gchar *) * (N_PLUGIN_DIRS+1));
-    plugin_dirs[0] = g_build_path(G_DIR_SEPARATOR_S, ".local", "lib",
-            "xfmedia", "plugins", NULL);
-    plugin_dirs[1] = g_build_path(G_DIR_SEPARATOR_S, LIBDIR, "xfmedia",
-            "plugins", NULL);
-    plugin_dirs[2] = NULL;
+    plugin_dirs[i++] = g_build_path(G_DIR_SEPARATOR_S, xfce_get_homedir(),
+                                    ".local", "lib", "xfmedia", "plugins",
+                                    NULL);
+    plugin_dirs[i++] = g_build_path(G_DIR_SEPARATOR_S, LIBDIR, "xfmedia",
+                                    "plugins", NULL);
     
-    for(i = 0; plugin_dirs[i]; i++) {
+    for(i = 0; i < N_PLUGIN_DIRS; i++) {
         GDir *dir = g_dir_open(plugin_dirs[i], 0, NULL);
-        if(!dir)
+        if(!dir) {
+            g_free(plugin_dirs[i]);
             continue;
+        }
         
         while((file = g_dir_read_name(dir))) {
             GModule *gm;
@@ -486,7 +489,7 @@ xfmedia_plugins_load(gpointer mwin)  /* gpointer to avoid an include */
                 continue;
             
             g_snprintf(fullpath, PATH_MAX, "%s/%s", plugin_dirs[i], file);
-            gm = g_module_open(fullpath, 0);
+            gm = g_module_open(fullpath, G_MODULE_BIND_LOCAL);
             if(!gm)
                 continue;
             
@@ -522,9 +525,9 @@ xfmedia_plugins_load(gpointer mwin)  /* gpointer to avoid an include */
             g_object_set_data(G_OBJECT(plugin), "__xfmedia-plugin-gmodule", gm);
             xfmedia_plugins = g_list_prepend(xfmedia_plugins, plugin);
         }
+        
+        g_free(plugin_dirs[i]);
     }
-    
-    g_strfreev(plugin_dirs);
 }
 
 GList *
