@@ -126,7 +126,7 @@ char *optarg;
 
    On entry to `getopt', zero means this is the first call; initialize.
 
-   When `getopt' returns -1, this is the index of the first of the
+   When `getopt' returns -1, this is the idx of the first of the
    non-option elements that the caller should itself scan.
 
    Otherwise, `optind' communicates from one call to the next
@@ -251,8 +251,8 @@ extern int strlen (const char *);
 /* Handle permutation of arguments.  */
 
 /* Describe the part of ARGV that contains non-options that have
-   been skipped.  `first_nonopt' is the index in ARGV of the first of them;
-   `last_nonopt' is the index after the last of them.  */
+   been skipped.  `first_nonopt' is the idx in ARGV of the first of them;
+   `last_nonopt' is the idx after the last of them.  */
 
 static int first_nonopt;
 static int last_nonopt;
@@ -475,7 +475,7 @@ _getopt_initialize (argc, argv, optstring)
    resume the scan with the following option character or ARGV-element.
 
    If there are no more option characters, `getopt' returns -1.
-   Then `optind' is the index in ARGV of the first ARGV-element
+   Then `optind' is the idx in ARGV of the first ARGV-element
    that is not an option.  (The ARGV-elements have been permuted
    so that those that are not options now come last.)
 
@@ -510,7 +510,7 @@ _getopt_initialize (argc, argv, optstring)
    LONGOPTS is a vector of `struct option' terminated by an
    element containing a name which is zero.
 
-   LONGIND returns the index in LONGOPT of the long-named option found.
+   LONGIND returns the idx in LONGOPT of the long-named option found.
    It is only valid when a long-named option has been found by the most
    recent call.
 
@@ -600,7 +600,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 
       if (optind == argc)
     {
-      /* Set the next-arg-index to point at the non-options
+      /* Set the next-arg-idx to point at the non-options
          that we previously skipped, so the caller will digest them.  */
       if (first_nonopt != last_nonopt)
         optind = first_nonopt;
