@@ -92,6 +92,9 @@ GType xfmedia_playlist_get_type                  () G_GNUC_CONST;
 
 GtkWidget *xfmedia_playlist_new                  ();
 
+gboolean xfmedia_playlist_load_default_playlist  (XfmediaPlaylist *plist);
+gboolean xfmedia_playlist_save_default_playlist  (XfmediaPlaylist *plist);
+
 guint xfmedia_playlist_append_entry              (XfmediaPlaylist *plist,
                                                   const gchar *title,
                                                   gint length,
@@ -126,6 +129,10 @@ gboolean xfmedia_playlist_get                    (XfmediaPlaylist *plist,
                                                   gchar **filename);
 guint xfmedia_playlist_get_n_entries             (XfmediaPlaylist *plist);
 
+void xfmedia_playlist_set_dirty                  (XfmediaPlaylist *plist,
+                                                  gboolean is_dirty);
+gboolean xfmedia_playlist_get_dirty              (XfmediaPlaylist *plist);
+
 gint xfmedia_playlist_get_selected               (XfmediaPlaylist *plist);
 
 void xfmedia_playlist_set_bold_entry             (XfmediaPlaylist *plist,
@@ -138,8 +145,6 @@ void xfmedia_playlist_context_menu_prepend_item  (XfmediaPlaylist *plist,
 void xfmedia_playlist_context_menu_append_item   (XfmediaPlaylist *plist,
                                                   GtkMenuItem *menu_item);
 
-gboolean xfmedia_playlist_is_dirty               (XfmediaPlaylist *plist);
-
 void xfmedia_playlist_set_shuffle_state          (XfmediaPlaylist *plist,
                                                   gboolean state);
 gboolean xfmedia_playlist_get_shuffle_state      (XfmediaPlaylist *plist);
