@@ -26,8 +26,13 @@
 
 G_BEGIN_DECLS
 
-XfmediaRemoteStatus xfmedia_remote_send_command(gint session_id, const gchar *command, ...);
-GList *xfmedia_remote_get_info(guint session_id, const gchar *command, ...);
+gboolean xfmedia_remote_client_init                   ();
+XfmediaRemoteStatus xfmedia_remote_client_send_command(gint session_id,
+                                                       const gchar *command,
+                                                       ...);
+GList *xfmedia_remote_client_get_info                 (guint session_id,
+                                                       const gchar *command,
+                                                       ...);
 
 G_END_DECLS
 
